
// This file is part of SPart, Spatially Located Workload Partitioner.

// Copyright (C) 2011, The Ohio State University
// SPart developed by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek 

// For questions, comments, suggestions, bugs please send e-mail to: 
// Umit V. Catalyurek   catalyurek.1@osu.edu

// SPart is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later
// version.

// SPart is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with SPart; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

// As a special exception, if other files instantiate templates or use
// macros or inline functions from this file, or you compile this file
// and link it with other works to produce a work based on this file,
// this file does not by itself cause the resulting work to be covered by
// the GNU General Public License. However the source code for this file
// must still be made available in accordance with section (3) of the GNU
// General Public License.

// This exception does not invalidate any other reasons why a work based
// on this file might be covered by the GNU General Public License.

// The full text of the GPL license version 2 is available in "gpl.txt".

/**
 * @file   direct_cut.hpp
 * @author erdeniz <erdeniz@bmi.osu.edu>
 * @date   Mon Jun 14 04:08:47 2010
 * 
 * @brief  Greedily allocates to each processor the smallest interval with load less than average load
 * 
 * 
 */


#ifndef __DIRECT_CUT_H__
#define __DIRECT_CUT_H__

#include <util/prefix_sum_tools.hpp>
#include <assert.h>
#include <stdlib.h>
#include <stdio.h>

namespace oned
{
/**
 * @brief Greedily allocates to each processor the smallest interval with load less than average load.
 *
 * This algorithm was introduced in "Serge Miguet and Jean-Marc
 * Pierson, Heuristics for 1D Rectilinear Partitioning as a Low Cost
 * and High Quality Answer to Dynamic Load Balancing, HPCN Europe
 * 1997" under the name H1. It sets cut points where B=P_iter*(Wtot/P).
 *
 * @param T data type of weight of instance @param Pr data type of
 * prefixSumArray
 */
template <typename T, typename Pr>
class DirectCut
{
  static T direct_cut_internal(int procCount, const Pr& prefixSumArray,
			       int length, int *cutIndexes);

public:
  /** 
   * 
   * @brief Applies direct cut algorithm to a given prefixSumArray
   * @param[in] procCount is the total number of processors
   * @param[in] prefixSumArray always begins with 0 as first element.
   * @param[in] length is the exact size of prefixSumArray
   * @param[out] cutIndexes (must be allocated before calling!)
   * @return Load of the max. loaded processor
   */
  static T direct_cut(int procCount, const Pr& prefixSumArray,
		      int length, int *cutIndexes, T);
  
};
}
#include <oned/direct_cut_impl.hpp>

#endif
