
// This file is part of SPart, Spatially Located Workload Partitioner.

// Copyright (C) 2011, The Ohio State University
// SPart developed by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek 

// For questions, comments, suggestions, bugs please send e-mail to: 
// Umit V. Catalyurek   catalyurek.1@osu.edu

// SPart is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later
// version.

// SPart is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with SPart; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

// As a special exception, if other files instantiate templates or use
// macros or inline functions from this file, or you compile this file
// and link it with other works to produce a work based on this file,
// this file does not by itself cause the resulting work to be covered by
// the GNU General Public License. However the source code for this file
// must still be made available in accordance with section (3) of the GNU
// General Public License.

// This exception does not invalidate any other reasons why a work based
// on this file might be covered by the GNU General Public License.

// The full text of the GPL license version 2 is available in "gpl.txt".

/**
 * @file   greedy_bisect.hpp
 * @author erdeniz <erdeniz@bmi.osu.edu>
 * @date   Mon Jun 14 06:04:55 2010
 * 
 * @brief Grabs the interval with largest load and cuts it into two
 * equal loaded pieces.
 * 
 * 
 */

#ifndef __GREEDY_BISECT_H__
#define __GREEDY_BISECT_H__

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <oned/rec_bisect.hpp>
#include <queue>
#include <algorithm>

namespace oned
{
/**
 * @brief Grabs the interval with largest load and cuts it into two
 * equal loaded pieces.
 *
 * @param T data type of weight of instance
 * @param Pr data type of prefixSumArray
 */
template <typename T, typename Pr>
class GreedyBisection : public RecursiveBisection<T,Pr>
{
  /// @cond DEV
  struct HeapElement
  {
    int low, high;
    T weight;
  };

  struct HeapElementCompare
  {
    bool operator() (const HeapElement &a, const HeapElement &b)
    {
      return a.weight<b.weight;
    }
  };
  /// @endcond
  static T greedy_bisect_internal(int procCount, const Pr& prefixSumArray,
				  int length, int *cutIndexes);

public:
  /** 
   * 
   * Applies the greedy bisection algorithm to a given prefixSumArray.
   * @param[in] procCount is the total number of processors.
   * @param[in] prefixSumArray always begins with 0 as first element.  
   * @param[in] length is the exact size of prefixSumArray.
   * @param[out] cutIndexes array of cut points.
   * @param foo maximum value of the array. This parameter is included for compatibility reason, it is ignored by the algorithm.
   * 
   * @return Load of the max. loaded processor
   */
  static T greedy_bisection(int procCount, const Pr& prefixSumArray,int length,
			    int *cutIndexes, T foo);
};
};
#include <oned/greedy_bisect_impl.hpp>

#endif
