
// This file is part of SPart, Spatially Located Workload Partitioner.

// Copyright (C) 2011, The Ohio State University
// SPart developed by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek 

// For questions, comments, suggestions, bugs please send e-mail to: 
// Umit V. Catalyurek   catalyurek.1@osu.edu

// SPart is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later
// version.

// SPart is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with SPart; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

// As a special exception, if other files instantiate templates or use
// macros or inline functions from this file, or you compile this file
// and link it with other works to produce a work based on this file,
// this file does not by itself cause the resulting work to be covered by
// the GNU General Public License. However the source code for this file
// must still be made available in accordance with section (3) of the GNU
// General Public License.

// This exception does not invalidate any other reasons why a work based
// on this file might be covered by the GNU General Public License.

// The full text of the GPL license version 2 is available in "gpl.txt".

/**
 * @file   nicol_minus.hpp
 * @author erdeniz <erdeniz@bmi.osu.edu>
 * @date   Mon Jun 14 10:38:41 2010
 * 
 * @brief <B>[Use NicolPlus instead]</B> this is the implementation of
 * the nicol's algorithm as it is provided in "Ali Pinar and Cevdet
 * Aykanat, Fast Optimal Load Balancing Algorithms for 1D
 * Partitioning, JPDC, 2004" in figure 5a.
 *
 * Applies binary search over bottleneck value and returns the least feasible bottleneck.
 *
 * 
 * 
 */


/*
 * (C) Ohio State University Biomedical Informatics - HPC Lab. http://www.bmi.osu.edu/hpc
 * 
 * 02/16/2010
 * nicol_minus.h
 *
 *
 */

#ifndef __NICOL_MINUS_H__
#define __NICOL_MINUS_H__

#include <util/prefix_sum_tools.hpp>
#include <util/parametric_search.hpp>
#include <stdlib.h>

namespace oned
{
  /**
   * @brief <B>[Use NicolPlus instead]</B> this is the implementation
   * of the nicol's algorithm as it is provided in "Ali Pinar and
   * Cevdet Aykanat, Fast Optimal Load Balancing Algorithms for 1D
   * Partitioning, JPDC, 2004" in figure 5a. NicolPlus should be
   * preferred over NicolMinus due to speed.
   */
template <typename T, typename Pr>
class NicolMinus
{

  //w: Prefix sum array that always starts with value 0 in the 0th
  //index, n is the length of w and p is the number of processors
  static T nicol_minus_internal(const Pr& w, int n, int p);

public:
  /** 
   * 
   * 
   * @param[in] procCount number of processors 
   * @param[in] prefixSumArray Prefix sum array that always starts with value 0 in the 0th index
   * @param[in] length is the length of w 
   * @param[out] cutIndexes cut index points
   * @param[in] max maximum load of a given array. If unknown, use -1.
   * 
   * @return Load of the max. loaded processor
   */
  static T nicol_minus(int procCount, const Pr& prefixSumArray,
		       int length, int *cutIndexes, T max);
};
};
#include <oned/nicol_minus_impl.hpp>

#endif
