
// This file is part of SPart, Spatially Located Workload Partitioner.

// Copyright (C) 2011, The Ohio State University
// SPart developed by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek 

// For questions, comments, suggestions, bugs please send e-mail to: 
// Umit V. Catalyurek   catalyurek.1@osu.edu

// SPart is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later
// version.

// SPart is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with SPart; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

// As a special exception, if other files instantiate templates or use
// macros or inline functions from this file, or you compile this file
// and link it with other works to produce a work based on this file,
// this file does not by itself cause the resulting work to be covered by
// the GNU General Public License. However the source code for this file
// must still be made available in accordance with section (3) of the GNU
// General Public License.

// This exception does not invalidate any other reasons why a work based
// on this file might be covered by the GNU General Public License.

// The full text of the GPL license version 2 is available in "gpl.txt".

/**
 * @file   nicol_plus_interval.hpp
 * @author erdeniz <erdeniz@bmi.osu.edu>
 * @date   Fri Jun 10 02:17:54 2011
 * 
 * @brief Given a prefix sum array in interval notation, this
 * algorithm applies NicolPlus.
 *
 * This is class is used by twod::RectNicol and
 * twod::JagPQOptIntervalBest.
 * 
 * 
 */


#ifndef __NICOL_PLUS_INTERVAL_H__
#define __NICOL_PLUS_INTERVAL_H__

#include <util/parametric_search.hpp>
namespace oned
{
  /**
   * @brief Given a prefix sum array in interval notation, this
   * algorithm applies NicolPlus. 
   *
   * This is class is used by twod::RectNicol and
   * twod::JagPQOptIntervalBest.
   * @param T data type of weight of instance
   * @param Pr data type of prefixSumArray
   */
  template <typename T, typename Pr>
  class Nicol_plus_interval
  {
    static T nicol_plus_internal(const Pr& wpre, int length,
				 int numproc, T wmax, int *sl, int *sh);
  public:
  /** 
   * 
   * 
   * @param[in] procCount number of processors 
   * @param[in] prefixSumArray Prefix sum array that always starts with value 0 in the 0th index
   * @param[in] length is the length of prefixSumArray
   * @param[out] cutIndexes cut index points
   * @param[in] max maximum load of a given array. If unknown, use -1.
   * 
   * @return Load of the max. loaded processor
   */
    static T nicol_plus(int procCount, const Pr& prefixSumArray,
			int length, int *cutIndexes, T max);
  };

}
#include <oned/nicol_plus_interval_impl.hpp>
#endif
