
// This file is part of SPart, Spatially Located Workload Partitioner.

// Copyright (C) 2011, The Ohio State University
// SPart developed by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek 

// For questions, comments, suggestions, bugs please send e-mail to: 
// Umit V. Catalyurek   catalyurek.1@osu.edu

// SPart is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later
// version.

// SPart is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with SPart; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

// As a special exception, if other files instantiate templates or use
// macros or inline functions from this file, or you compile this file
// and link it with other works to produce a work based on this file,
// this file does not by itself cause the resulting work to be covered by
// the GNU General Public License. However the source code for this file
// must still be made available in accordance with section (3) of the GNU
// General Public License.

// This exception does not invalidate any other reasons why a work based
// on this file might be covered by the GNU General Public License.

// The full text of the GPL license version 2 is available in "gpl.txt".

#include <oned/nicol_plus_interval.hpp>


template <typename T, typename Pr>
T oned::Nicol_plus_interval<T,Pr>::nicol_plus_internal(const Pr& wpre, int length,
						       int numproc, T , int *sl, int *sh)
{
  using namespace util;
  int ilast = 1, b;
  int ilow, ihigh, imid;
  T bottleneck, Bopt;

  T lb = 0; // there is no good lower bound if the intervals does not respect load(I_a \union I_b) = load(I_a)+load(I_b)
  T ub = wpre.interval(0,length - 1);
  int *s = new int[numproc];
  int *oris = s;
  Bopt = ub;

  for(b=0; b < numproc - 1; b++)
    {

      ilow = (sl[b]<ilast?ilast-1:sl[b]);
      ihigh=length-1;
      assert (ilow <= ihigh);

      while(ilow < ihigh)
	{
	  imid = (ilow + ihigh) / 2;
	  bottleneck = wpre.interval(ilast - 1,imid);
	  assert (bottleneck >=0);
	  if((lb <= bottleneck) && (bottleneck < ub))
	    {
	      if(ParametricSearch<T,Pr>::rprobe_interval(wpre, length, bottleneck, numproc, s, sl, sh))
		{
		  int *tmp;
		  tmp=s;
		  s=sh;
		  sh=tmp;
		  ub = bottleneck;
		  ihigh = imid;
		}
	      else
		{
		  int *tmp;
		  tmp=s;
		  s=sl;
		  sl=tmp;
		  lb = bottleneck;
		  ilow = imid + 1;
		}
	    }
	  else if(bottleneck >= ub)
	    {
	      ihigh = imid;
	    }
	  else
	    {
	      ilow = imid + 1;
	    }
	}
      assert (ilow == ihigh);
      assert(b < numproc);

      Bopt = std::min(Bopt, wpre.interval(ilast - 1, ihigh)); //if b is the bottleneck, it executes [ilast-1:high]
      ilast = ihigh;
    }

  Bopt = std::min(Bopt, wpre.interval(ilast-1,length-1));
  delete[] oris;
  return Bopt;
}


template <typename T, typename Pr>
T oned::Nicol_plus_interval<T,Pr>::nicol_plus(int procCount, const Pr& prefixSumArray,
					      int length, int *cutIndexes, T max)
{
  using namespace util;
  int i;
  int *sl =  new int[procCount];
  int *sh =  new int[procCount];
  T b;

  for(i = 0; i < procCount; i++)
    {
      sl[i] = 1;
      sh[i] = length - 1;
    }

  b = nicol_plus_internal (prefixSumArray, length, procCount, max, sl, sh);

  if (cutIndexes != NULL)
    {
#ifndef NDEBUG
      int valid = ParametricSearch<T,Pr>::probeCut_interval(b, procCount - 1, prefixSumArray, length, cutIndexes);
      assert(valid==1);
#endif
#ifdef NDEBUG
      ParametricSearch<T,Pr>::probeCut_interval(b, procCount - 1, prefixSumArray, length, cutIndexes);
#endif
    }
  else
    {
      std::cerr << "Error: cutIndexes not allocated in nicol plus 2d!" << std::endl;
    }

  delete[] sl;
  delete[] sh;
  return b;
}

