// This file is part of SPart, Spatially Located Workload Partitioner.

// Copyright (C) 2011, The Ohio State University
// SPart developed by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek 

// For questions, comments, suggestions, bugs please send e-mail to: 
// Umit V. Catalyurek   catalyurek.1@osu.edu

// SPart is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later
// version.

// SPart is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with SPart; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

// As a special exception, if other files instantiate templates or use
// macros or inline functions from this file, or you compile this file
// and link it with other works to produce a work based on this file,
// this file does not by itself cause the resulting work to be covered by
// the GNU General Public License. However the source code for this file
// must still be made available in accordance with section (3) of the GNU
// General Public License.

// This exception does not invalidate any other reasons why a work based
// on this file might be covered by the GNU General Public License.

// The full text of the GPL license version 2 is available in "gpl.txt".

/**
 * @file oned/parse.hpp
 * 
 * @brief  Converts the name of a 1d algorithm into a function pointer
 * 
 * 
 */


#ifndef __PARSE_1D_HPP__
#define __PARSE_1D_HPP__
#include <string>
#include "oned/nicol.hpp"
#include "oned/nicol_minus.hpp"
#include "oned/rec_bisect.hpp"
#include "oned/greedy_bisect.hpp"
#include "oned/direct_cut.hpp"
#include "oned/direct_ref_b.hpp"
#include "oned/nicol_plus.hpp"
#include "util/version.hpp"

namespace oned
{
  /**
   * @brief Converts text into corresponding 1D algorithm function pointer.
   * 
   */

  template<typename T, typename Pr>
  class Parser
  {
  public:
    /**
     * 
     * @brief Given a 1D algorithm name string, returns proper
     * partitioning function pointer.
     * 
     * @param name Algorithm name. Currently valid names are: NICOL-MINUS, NICOL, NICOL-PLUS, REC-BISECT, GREEDY-BISECT, DIRECT-CUT, DIRECT-CUT-REF-B
     * 
     * @return function pointer corresponding to name
     */
    static T (*parseName(const std::string& name))(int, Pr const&,int, int *, T);
  };
}

template<typename T, typename Pr>
T (*oned::Parser<T,Pr>::parseName(const std::string& name))(int, Pr const&,int, int *, T)
{
  //select algorithm
  if(name == "NICOL-MINUS")
    {
      return oned::NicolMinus<T, Pr>::nicol_minus;
    }
  else if(name == "NICOL")
    {
      return oned::Nicol<T, Pr >::nicol;
    }
  else if(name == "NICOL-PLUS")
    {
      return oned::NicolPlus<T, Pr>::nicol_plus;
    }
  else if(name == "REC-BISECT")
    {
      return oned::RecursiveBisection<T, Pr>::rec_bisection;
    }
  else if(name == "GREEDY-BISECT")
    {
      return oned::GreedyBisection<T,Pr >::greedy_bisection;
    }
  else if(name == "DIRECT-CUT")
    {
      return oned::DirectCut<T, Pr >::direct_cut;
    }
  else if(name == "DIRECT-CUT-REF-B")
    {
      return oned::DirectCutRefB<T, Pr >::direct_ref_b;
    }
  else return NULL;
}

#endif
