
// This file is part of SPart, Spatially Located Workload Partitioner.

// Copyright (C) 2011, The Ohio State University
// SPart developed by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek 

// For questions, comments, suggestions, bugs please send e-mail to: 
// Umit V. Catalyurek   catalyurek.1@osu.edu

// SPart is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later
// version.

// SPart is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with SPart; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

// As a special exception, if other files instantiate templates or use
// macros or inline functions from this file, or you compile this file
// and link it with other works to produce a work based on this file,
// this file does not by itself cause the resulting work to be covered by
// the GNU General Public License. However the source code for this file
// must still be made available in accordance with section (3) of the GNU
// General Public License.

// This exception does not invalidate any other reasons why a work based
// on this file might be covered by the GNU General Public License.

// The full text of the GPL license version 2 is available in "gpl.txt".

// this file is given for documentation purpose only and should not be compiled

/**
 * @brief This namespace provides one dimensional algorithms.
 *
 * This namespace provides one dimensional algorithms. Both optimal
 * and heuristic algorithms are implemented.  All algorithms except
 * Nicol variants are heuristic. Nicol has 3 variants which generate
 * the same output but use different bounding tecniques. We recommend
 * using NicolPlus since it is optimal and fast. One should avoid
 * using NicolMinus or Nicol variants and prefer NicolPlus. If you
 * really need use heuristics to improve speed, we recommend DirectCut
 * or RecursiveBisection.
 *
 * The resulting cut points map to the prefix sum array indexes and
 * exclude leftmost and rightmost cut points, which are constant (0
 * and arraysize-1). Elements with the same index as cut point value
 * are included to cut points' corresponding processor.  For example
 * consider a prefix sum array with elements {0,5,7,15,21,40,52} (correspond to original array with elements {5,2,8,6,19,12}), P=3
 * and cut points {3,5}. Then processor P0 has {5,2,8}, P1 has {6,19}
 * and P3 has {12} as actual workload. Notice prefix sum array always
 * has 0 as its 1st element. The length of cut point array is always
 * P-1.
 *
 * All algorithms except Nicol_plus_interval use bracket notation as
 * defined in util::PrefixSumArray1DOfTBracket. Nicol_plus_interval
 * uses interval nototaion as defined in util::PrefixSumArray1DOfTInterval.
 */
namespace oned
{
}
