
// This file is part of SPart, Spatially Located Workload Partitioner.

// Copyright (C) 2011, The Ohio State University
// SPart developed by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek 

// For questions, comments, suggestions, bugs please send e-mail to: 
// Umit V. Catalyurek   catalyurek.1@osu.edu

// SPart is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later
// version.

// SPart is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with SPart; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

// As a special exception, if other files instantiate templates or use
// macros or inline functions from this file, or you compile this file
// and link it with other works to produce a work based on this file,
// this file does not by itself cause the resulting work to be covered by
// the GNU General Public License. However the source code for this file
// must still be made available in accordance with section (3) of the GNU
// General Public License.

// This exception does not invalidate any other reasons why a work based
// on this file might be covered by the GNU General Public License.

// The full text of the GPL license version 2 is available in "gpl.txt".

#ifndef __UNI_ONE__
#define __UNI_ONE__
#include <util/prefix_sum_tools.hpp>

namespace oned
{
  /**
   * @brief Cuts prefix sum array into p fixed and equal length parts
   * where p is the processor number
   * 
   */
  template<typename T, typename Pr>
  class Uniform
  {
  public:
    /** 
     * @brief Cuts prefix sum array into p fixed and equal length
     * parts where p is the processor number
     * 

     * @param[in] procCount is the number of processors
     * @param[in] prefixSumArray Prefix sum array that always starts with value 0 in the 0th
     * @param[in] length is the length of prefixSumArray
     * @param[out] cutIndexes
     * @param[in] max is the load of the most loaded cell of the array
     * (-1 if unknown). Unused in this algorithm.
     * 
     * @return maximum loaded processor's load
     */
    static T uni_cut(int procCount, const Pr& prefixSumArray, int length, int *cutIndexes, T max);
  };
}
#include <oned/uniform_impl.hpp>
#endif
