
// This file is part of SPart, Spatially Located Workload Partitioner.

// Copyright (C) 2011, The Ohio State University
// SPart developed by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek 

// For questions, comments, suggestions, bugs please send e-mail to: 
// Umit V. Catalyurek   catalyurek.1@osu.edu

// SPart is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later
// version.

// SPart is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with SPart; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

// As a special exception, if other files instantiate templates or use
// macros or inline functions from this file, or you compile this file
// and link it with other works to produce a work based on this file,
// this file does not by itself cause the resulting work to be covered by
// the GNU General Public License. However the source code for this file
// must still be made available in accordance with section (3) of the GNU
// General Public License.

// This exception does not invalidate any other reasons why a work based
// on this file might be covered by the GNU General Public License.

// The full text of the GPL license version 2 is available in "gpl.txt".

#include <oned/uniform.hpp>

template<typename T, typename Pr>
T oned::Uniform<T,Pr>::uni_cut(int procCount, const Pr& prefixSumArray, int length, int *cutIndexes, T)
{
  double lenpart=(length - 1)/((double)procCount);
  cutIndexes[0]=(int)lenpart;
  T max = prefixSumArray[cutIndexes[0]];
  for(int i = 1;i < procCount-1;i++)
    {
      cutIndexes[i]=((int)((i+1)*lenpart));
      assert(cutIndexes[i] < length);
      T pload=prefixSumArray[cutIndexes[i]] - prefixSumArray[cutIndexes[i-1]];
      max=std::max(pload,max);
    }
  max=std::max(max, prefixSumArray[length-1] - prefixSumArray[cutIndexes[procCount-2]]);
  return max;
}
