
// This file is part of SPart, Spatially Located Workload Partitioner.

// Copyright (C) 2011, The Ohio State University
// SPart developed by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek 

// For questions, comments, suggestions, bugs please send e-mail to: 
// Umit V. Catalyurek   catalyurek.1@osu.edu

// SPart is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later
// version.

// SPart is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with SPart; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

// As a special exception, if other files instantiate templates or use
// macros or inline functions from this file, or you compile this file
// and link it with other works to produce a work based on this file,
// this file does not by itself cause the resulting work to be covered by
// the GNU General Public License. However the source code for this file
// must still be made available in accordance with section (3) of the GNU
// General Public License.

// This exception does not invalidate any other reasons why a work based
// on this file might be covered by the GNU General Public License.

// The full text of the GPL license version 2 is available in "gpl.txt".

#ifndef __PART_BASE__H__
#define __PART_BASE__H__

#include <util/rect_list.hpp>
#include <util/prefix_sum_tools.hpp>

namespace twod
{
  /**
   * @brief base class for the 2D partitioning algorithm
   * @param T type of the element in the matrix
   * @param Pr type of the prefix sum array
   */
  template <typename T, typename Pr>
  class PartBase
  {
    
  public:
    /** 
     * 
     * @brief Applies a 2d partitioning algorithm to a given 2d prefixSumArray .
     * 
     * @param[in] procCount is the number of processors
     * @param[in] prefixSumArray first column and first row consists of zero only. But the borders of rectangles in rect_list never touch this area (index 0 in row or column)
     * @param[out] parts must be empty before calling this function. It will contain the partition after the function return
     * 
     * @return the load of the most loaded rectangle of parts
     */
    virtual T part(int procCount, const Pr& prefixSumArray, util::RectList<T,Pr> &parts)=0;
    virtual ~PartBase();
  };
}

#include <twod/part_base_impl.hpp>
#endif
