
// This file is part of SPart, Spatially Located Workload Partitioner.

// Copyright (C) 2011, The Ohio State University
// SPart developed by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek 

// For questions, comments, suggestions, bugs please send e-mail to: 
// Umit V. Catalyurek   catalyurek.1@osu.edu

// SPart is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later
// version.

// SPart is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with SPart; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

// As a special exception, if other files instantiate templates or use
// macros or inline functions from this file, or you compile this file
// and link it with other works to produce a work based on this file,
// this file does not by itself cause the resulting work to be covered by
// the GNU General Public License. However the source code for this file
// must still be made available in accordance with section (3) of the GNU
// General Public License.

// This exception does not invalidate any other reasons why a work based
// on this file might be covered by the GNU General Public License.

// The full text of the GPL license version 2 is available in "gpl.txt".

/**
 * @file   rec_bisect_2d.hpp
 * @author erdeniz <erdeniz@bmi.osu.edu>
 * @date   Mon Jun 14 21:05:17 2010
 * 
 * @brief  this is the implementation of 2D relaxed hierarchical heuristic
 * 
 * 
 */


/*
 * (C) Ohio State University Biomedical Informatics - HPC Lab. http://www.bmi.osu.edu/hpc
 * 
 * 04/04/2010
 * load_rec_bisect_2d.h
 *
 * this is the implementation of 2D relaxed hierarchical
 *
 */

#ifndef __REC_BISECT_MINMAX_2D_H__
#define __REC_BISECT_MINMAX_2D_H__

#include <util/prefix_sum_tools.hpp>
#include <util/rect_list.hpp>
namespace twod
{
  /**
   * @brief implements the hierarchical relaxed heuristic and some
   * variants.
   *
   * @param T is the data type of prefix sum array.
   * @param Pr is the 2D prefix sum array.
   * @param type is the recursive bisection strategy type. 
   * Strategy types are LOAD, DIST, VERT_ALT, HOR_ALT
   */

  template<typename T, typename Pr,  int type=0, bool middle=false>
  class HierRelaxed : public PartBase<T,Pr>
  {
  private:
    static T rec_bisect_2d_internal(int procCount, const Pr& prefixSumArray, util::RectList<T,Pr> &parts, util::rectangle r, bool orient);
  public:  

    virtual T part(int procCount, const Pr& prefixSumArray, util::RectList<T,Pr> &parts);
    virtual ~HierRelaxed(){}

    ///Cut to minimize load
    static const int LOAD=0;
    ///Cut to minimize side-to-side distance
    static const int DIST=1;
    ///start vertically and alternate between horizontal
    static const int VERT_ALT=2;
    ///start horizontally and alternate between vertical
    static const int HOR_ALT=3;

  };

}
#include <twod/rec_bisect_relaxed_impl.hpp>

#endif



