
// This file is part of SPart, Spatially Located Workload Partitioner.

// Copyright (C) 2011, The Ohio State University
// SPart developed by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek 

// For questions, comments, suggestions, bugs please send e-mail to: 
// Umit V. Catalyurek   catalyurek.1@osu.edu

// SPart is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later
// version.

// SPart is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with SPart; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

// As a special exception, if other files instantiate templates or use
// macros or inline functions from this file, or you compile this file
// and link it with other works to produce a work based on this file,
// this file does not by itself cause the resulting work to be covered by
// the GNU General Public License. However the source code for this file
// must still be made available in accordance with section (3) of the GNU
// General Public License.

// This exception does not invalidate any other reasons why a work based
// on this file might be covered by the GNU General Public License.

// The full text of the GPL license version 2 is available in "gpl.txt".

#include <assert.h>
#include <twod/rec_bisect_relaxed.hpp>
#include <oned/rec_bisect_minmax.hpp>
#include <util/tools.hpp>
#include <iostream>

template<typename T, typename Pr, int type, bool middle>
T twod::HierRelaxed<T,Pr,type,middle>::part(int procCount, const Pr& prefixSumArray, util::RectList<T,Pr> &parts)
 {
   util::rectangle r;
   bool orient;
   if(type == HOR_ALT)
     orient = true;
   else
     orient = false;
   r.x_top_l = 1;
   r.y_top_l = 1;
   r.x_bot_r = prefixSumArray.prefixsizeX() - 1;
   r.y_bot_r = prefixSumArray.prefixsizeY() - 1;
   return rec_bisect_2d_internal(procCount, prefixSumArray, parts, r, orient);
 }

template<typename T, typename Pr,  int type, bool middle>
T twod::HierRelaxed<T,Pr,type,middle>::rec_bisect_2d_internal(int procCount, const Pr& prefixSumArray, util::RectList<T,Pr> &parts, util::rectangle r, bool orient)
{
  using namespace oned;
  using namespace util;
  //there is a cut to perform (more than one cell and one processor)

  bool can_vertical_cut;
  bool can_horizontal_cut;
  
  if (type == LOAD)
    {
      can_vertical_cut = true;
      can_horizontal_cut = true;

    }
  if (type == DIST)
    {
      if ((r.x_bot_r - r.x_top_l)
	  <= (r.y_bot_r - r.y_top_l))
	{
	  can_vertical_cut = true;
	  can_horizontal_cut = false;
	}
      else
	{
	  can_vertical_cut = false;
	  can_horizontal_cut = true;
	}
    }

  if((type == HOR_ALT) || (type == VERT_ALT))
    {
      if (orient)
	{
	  can_vertical_cut = false;
	  can_horizontal_cut = true;
	  //if the dimension to cut is composed of a single cell, swap the dimensions
	  if (r.x_top_l == r.x_bot_r) 
	    {
	      can_horizontal_cut = false;
	      can_vertical_cut = true;
	    }
	}
      
      else
	{
	  can_vertical_cut = true;
	  can_horizontal_cut = false;
	  //if the dimension to cut is composed of a single cell, swap the dimensions
	  if (r.y_top_l == r.y_bot_r)
	    {
	      can_vertical_cut = false;
	      can_horizontal_cut = true;
	  
	    }

	}
    }
  
  if (r.y_top_l == r.y_bot_r)
    can_vertical_cut = false;
  
  if (r.x_top_l == r.x_bot_r)
    can_horizontal_cut = false; 
  

  if(procCount > 1 && (can_horizontal_cut || can_vertical_cut))
    {
      double ver_max=-1, hor_max=-1;
      int cutIndex_ver=-1, tmp_cutIndex_ver, cutIndex_hor=-1,tmp_cutIndex_hor;
      rectangle subr1_ver = r, subr2_ver = r;
      rectangle subr1_hor = r, subr2_hor = r;
      int leftProc = 1;
      int rightProc = procCount - leftProc;
      int tmpleft, tmpright, left_ver=-1, right_ver=-1, left_hor=-1, right_hor=-1;
      

      //vertical estimation
      if (can_vertical_cut)
	{
	  Aggreg2Dto1D<T, Pr, false> ps1d_ver(prefixSumArray, r.x_top_l, r.x_bot_r, r.y_top_l, r.y_bot_r);
	  if(middle)
	    //if middle constraint is given cut from the middle
	    tmp_cutIndex_ver = (r.y_bot_r - r.y_top_l)/2 + r.y_top_l;
	  else
	    tmp_cutIndex_ver = r.y_top_l-1 + RecursiveBisectionMinMax<T, Aggreg2Dto1D<T,  Pr, false>& >::findEvenCut(ps1d_ver, 1, r.y_bot_r-r.y_top_l+1, leftProc, rightProc);

	  cutIndex_ver = tmp_cutIndex_ver;
	  

	  assert (tmp_cutIndex_ver >= r.y_top_l);
	  assert (tmp_cutIndex_ver < r.y_bot_r);


	  double tmp_candDiff = RecursiveBisectionMinMax<T, Aggreg2Dto1D<T,  Pr, false>& >::calculateDiff(ps1d_ver, 1, r.y_bot_r-r.y_top_l+1, leftProc, rightProc, cutIndex_ver - (r.y_top_l-1));
	  double candDiff = tmp_candDiff;

	  for(tmpleft = leftProc + 1; tmpleft < procCount; tmpleft++)
	    {
	      tmpright = procCount - tmpleft;
	      if(!middle)
     		tmp_cutIndex_ver = r.y_top_l-1 + RecursiveBisectionMinMax<T, Aggreg2Dto1D<T,  Pr, false>& >::findEvenCut(ps1d_ver, 1, r.y_bot_r-r.y_top_l+1, tmpleft, tmpright); 
	      tmp_candDiff = RecursiveBisectionMinMax<T, Aggreg2Dto1D<T,  Pr, false>& >::calculateDiff(ps1d_ver, 1, r.y_bot_r - r.y_top_l+1, tmpleft, tmpright, tmp_cutIndex_ver - (r.y_top_l-1));

	      assert (tmp_cutIndex_ver >= r.y_top_l);
	      assert (tmp_cutIndex_ver < r.y_bot_r);

	      if(tmp_candDiff < candDiff)
		{
		  leftProc = tmpleft;
		  candDiff = tmp_candDiff;
		  cutIndex_ver = tmp_cutIndex_ver;
		}
	    }
	  rightProc = procCount - leftProc;
	  ver_max = candDiff;
	  
	  left_ver = leftProc;
	  right_ver = rightProc;


	  assert (cutIndex_ver >= 0);
	  
	  assert (cutIndex_ver >= r.y_top_l);
	  assert (cutIndex_ver < r.y_bot_r);

	  //	  assert (cutIndex_ver + 1 > r.y_top_l);
	  //	  assert (cutIndex_ver + 1 <= r.y_bot_r);

	  subr1_ver.y_bot_r = cutIndex_ver;
	  //	  subr2_ver.y_top_l = (cutIndex_ver + 1 > r.y_top_l ? r.y_top_l : cutIndex_ver + 1);
	  subr2_ver.y_top_l = cutIndex_ver + 1;
	}
      leftProc = 1;
      rightProc = procCount - leftProc;
      //horizontal estimation
      if (can_horizontal_cut)
	{
	  Aggreg2Dto1D<T, Pr, true> ps1d_hor(prefixSumArray, r.x_top_l, r.x_bot_r, r.y_top_l, r.y_bot_r); 	
	  if(middle)
	    tmp_cutIndex_hor = (r.x_bot_r - r.x_top_l)/2 + r.x_top_l;
	  else
	    tmp_cutIndex_hor = r.x_top_l-1 + RecursiveBisectionMinMax<T, Aggreg2Dto1D<T,  Pr, true>& >::findEvenCut(ps1d_hor, 1, r.x_bot_r-r.x_top_l+1, leftProc, rightProc);
	  cutIndex_hor = tmp_cutIndex_hor;

	  assert (tmp_cutIndex_hor >= r.x_top_l);
	  assert (tmp_cutIndex_hor <  r.x_bot_r);


	  double tmp_candDiff = RecursiveBisectionMinMax<T, Aggreg2Dto1D<T,  Pr, true>& >::calculateDiff(ps1d_hor, 1, r.x_bot_r-r.x_top_l+1, leftProc, rightProc, cutIndex_hor-(r.x_top_l-1));
	  double candDiff = tmp_candDiff;
	  for(tmpleft = leftProc + 1; tmpleft < procCount; tmpleft++)
	    {
	      tmpright = procCount - tmpleft;
	      if(!middle)
		tmp_cutIndex_hor = r.x_top_l-1 + RecursiveBisectionMinMax<T, Aggreg2Dto1D<T,  Pr, true>& >::findEvenCut(ps1d_hor, 1, r.x_bot_r-r.x_top_l+1, tmpleft, tmpright);
	      tmp_candDiff = RecursiveBisectionMinMax<T, Aggreg2Dto1D<T,  Pr, true>& >::calculateDiff(ps1d_hor, 1, r.x_bot_r-r.x_top_l+1, tmpleft, tmpright, cutIndex_hor-(r.x_top_l-1));

	      assert (tmp_cutIndex_hor >= r.x_top_l);
	      assert (tmp_cutIndex_hor <  r.x_bot_r);


	       if(tmp_candDiff < candDiff)
		{
		  leftProc = tmpleft;
		  candDiff = tmp_candDiff;
		  cutIndex_hor = tmp_cutIndex_hor;
		}
	    }
	  rightProc = procCount - leftProc;
	  hor_max = candDiff;

	  left_hor = leftProc;
	  right_hor = rightProc;

	  assert(cutIndex_hor >= 0);
	  
	  assert (cutIndex_hor >= r.x_top_l);
	  assert (cutIndex_hor <  r.x_bot_r);
	  //	  assert (cutIndex_hor + 1 >= r.x_top_l);
	  //	  assert (cutIndex_hor + 1 <= r.x_bot_r);

	  subr1_hor.x_bot_r = cutIndex_hor;
	  //	  subr2_hor.x_top_l = (cutIndex_hor + 1 > r.x_bot_r ? r.x_bot_r : cutIndex_hor + 1);
	  subr2_hor.x_top_l = cutIndex_hor + 1;
	}

      //decision
      T load1, load2;
      if( can_vertical_cut && ((!can_horizontal_cut ) || ver_max<hor_max))
	{

	  assert (leftProc + rightProc <= procCount);
	  //vertical cutting
	  
	  load1 = HierRelaxed::rec_bisect_2d_internal(left_ver, prefixSumArray, parts, subr1_ver, !orient);
	  if(cutIndex_ver + 1 < prefixSumArray.prefixsizeY())
	    load2 = HierRelaxed::rec_bisect_2d_internal(right_ver, prefixSumArray, parts, subr2_ver, !orient);
	  else
	    load2 = load1;
	}
      else
	{
	  //horizontal cutting
	  assert (can_horizontal_cut);
	  assert (leftProc + rightProc <= procCount);

	  load1 = HierRelaxed::rec_bisect_2d_internal(left_hor, prefixSumArray, parts, subr1_hor, !orient);
	  if(cutIndex_hor + 1 < prefixSumArray.prefixsizeX())
	    load2 = HierRelaxed::rec_bisect_2d_internal(right_hor, prefixSumArray, parts, subr2_hor, !orient);
	  else
	    load2 = load1;
	  
	}

      return  std::max(load1,load2);
    }
  else
    {

      parts.add_rect(r);
      return r.get_load<T,Pr>(prefixSumArray);
    }
  
}
