
// This file is part of SPart, Spatially Located Workload Partitioner.

// Copyright (C) 2011, The Ohio State University
// SPart developed by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek 

// For questions, comments, suggestions, bugs please send e-mail to: 
// Umit V. Catalyurek   catalyurek.1@osu.edu

// SPart is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later
// version.

// SPart is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with SPart; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

// As a special exception, if other files instantiate templates or use
// macros or inline functions from this file, or you compile this file
// and link it with other works to produce a work based on this file,
// this file does not by itself cause the resulting work to be covered by
// the GNU General Public License. However the source code for this file
// must still be made available in accordance with section (3) of the GNU
// General Public License.

// This exception does not invalidate any other reasons why a work based
// on this file might be covered by the GNU General Public License.

// The full text of the GPL license version 2 is available in "gpl.txt".

// this file is given for documentation purpose only and should not be compiled

/**
 * @brief This namespace provides two dimensional algorithms.
 *
 * This namespace provides two dimensional algorithms. There are
 * different classes of algorithms that generate different partition
 * structures. \image html partclass.jpg "Different structures of partitions"
 *
 * Rectilinear partitioning algorithms are described in section 3.1,
 * jagged partitioning algorithms are in section 3.2, hierarchical
 * partitioning algorithms are in 3.3 and hybrid part. are in section
 * 5 of <a href="http://arxiv.org/abs/1104.2566"> "E. Saule,
 * E. O. Bas, U. V. Catalyurek, Load-Balancing Spatially Located
 * Computations using Rectangular Partitions Paper, Tech. Report,
 * ArXiv, no. arXiv:1104.2566v1, 2011" </a> and twod::Hybrid
 * doucmentation. The partition structures are denoted at the prefix
 * of each class' name. For example JagMHeurProbeBest generates jagged
 * partition structure. We see this form Jag- prefix. The other
 * prefixes are Hier- (Hierarchical) and Rect- (Rectilinear). Hybrid
 * algorithms generate a combination of those structures depending on
 * the algorithm used in each level.
 *
 * Most of 2D partitioning algorithms have 3 variants based on first
 * dimension's cut orientation. Those are denoted at the suffix of
 * each class' name such as: xxxHor, xxxVer, xxxBest. xxxHor and
 * xxxVer first places horizontal and vertical cuts respectively. For
 * example JagMHeurProbeHor first creates row-wise stripes and then
 * independently partitions each stripe. Similarly, JagMHeurProbeVer
 * creates column stripes. xxxBest tries both orientations and returns
 * the solution with lesser bottleneck (bottleneck refers to maximum
 * loaded processor's load).
 *
 * Optimal algorithms are provided in jagged partitioning. Those are:
 * JagMOptBest, JagPQOptBest, JagPQOptIntervalBest (or their variants).
 *
 *
 * However, optimal algorithms can take a very long time. Therefore we
 * recommend using effective heuristic algorithms such as JagMHeurBest or
 * JagMHeurProbeBest (or their variants).
 *
 * The solution of 2D algorithms are expressed using util::RectList and
 * util::rectangle.
 *
 * All 2D algorithms derive from PartBase class. twod::Parser::parseName()
 * function is used to easily translate string name into proper
 * object. All 2D algorithms use bracket notation for 2D prefix sum
 * array as noted in util::PrefixSumArray2DOfTBracket.
 *
 * Given a 2D algorithm name string, twod::Parser::parseName() returns proper
 * partitioning algorithm object. All algorithms can be found and
 * conveniently used in this function. If a new algorithm is
 * implemented, it should derive from PartBase class and the algorithm
 * may be added to twod::Parser::parseName() for easy string-to-object
 * conversion.
 */
namespace twod
{
}
