
// This file is part of SPart, Spatially Located Workload Partitioner.

// Copyright (C) 2011, The Ohio State University
// SPart developed by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek 

// For questions, comments, suggestions, bugs please send e-mail to: 
// Umit V. Catalyurek   catalyurek.1@osu.edu

// SPart is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later
// version.

// SPart is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with SPart; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

// As a special exception, if other files instantiate templates or use
// macros or inline functions from this file, or you compile this file
// and link it with other works to produce a work based on this file,
// this file does not by itself cause the resulting work to be covered by
// the GNU General Public License. However the source code for this file
// must still be made available in accordance with section (3) of the GNU
// General Public License.

// This exception does not invalidate any other reasons why a work based
// on this file might be covered by the GNU General Public License.

// The full text of the GPL license version 2 is available in "gpl.txt".

#ifndef __UNIFORM_2D_H__
#define __UNIFORM_2D_H__

#include <twod/part_base.hpp>
#include <util/rect_list.hpp>
#include <util/prefix_sum_tools.hpp>

namespace twod
{
  /**
   * @brief Implements the area based rectilinear of the space.
   *
   * The number of processors in each dimension can be controlled by setP().
   * 
   * @param T type of element of the array
   * @param Pr type of the prefix sum array
   */
  template <typename T, typename Pr>
  class Uniform : public PartBase<T,Pr>
  {
    
    int P;/// number of part in the first dimension. if 0, automatic
  public:
    Uniform():P(0){}
    /**
     * @brief selects the number of stripes in the first dimension
     *
     * @param P number of stripes in the first dimension. If 0, the
     * number of stripes will be computed at runtime by taking the
     * square root of the number of processors. 
     */
    void setP(int P){this->P = P;}

    virtual T part(int procCount, const Pr& prefixSumArray, util::RectList<T,Pr> &parts);
    virtual ~Uniform(){}
  };
}
#include <twod/uniform_2d_impl.hpp>

#endif
