
// This file is part of SPart, Spatially Located Workload Partitioner.

// Copyright (C) 2011, The Ohio State University
// SPart developed by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek 

// For questions, comments, suggestions, bugs please send e-mail to: 
// Umit V. Catalyurek   catalyurek.1@osu.edu

// SPart is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later
// version.

// SPart is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with SPart; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

// As a special exception, if other files instantiate templates or use
// macros or inline functions from this file, or you compile this file
// and link it with other works to produce a work based on this file,
// this file does not by itself cause the resulting work to be covered by
// the GNU General Public License. However the source code for this file
// must still be made available in accordance with section (3) of the GNU
// General Public License.

// This exception does not invalidate any other reasons why a work based
// on this file might be covered by the GNU General Public License.

// The full text of the GPL license version 2 is available in "gpl.txt".

#ifndef __AGGREG_2D_TO_1D_PART
#define __AGGREG_2D_TO_1D_PART

namespace util
{
  /**
   * @brief This class implements the interval notation for 1d array
   * as defined by util::PrefixSumArray1DOfTInterval.
   *
   * This class implements the interval notation for 1d array as
   * defined by util::PrefixSumArray1DOfTInterval. The data come from
   * a 2d prefix sum array. The size of the exposed 1d array is given
   * by the first dimension of the 2d prefix sum array. When asked for
   * the value of an interval (in the first dimension), the stripe
   * defined by the interval is partitioned along the second dimension
   * using the oned::NicolPlus algorithm.
   *
   * @param T type of the data.
   * @param Pr type of the 2d prefix sum array.
   */
  template <typename T, typename Pr>
  class Aggreg2Dto1Dpart
  {
    ///the 2d prefix sum array which is represented
    const Pr& pr;
    const int m;
    int *temp;
  
  private:
    ///deactivated since it does not make sense
    Aggreg2Dto1Dpart<T,Pr>& operator=(const Aggreg2Dto1Dpart<T,Pr>& rhs);
    ///deactivated since it does not make sense
    Aggreg2Dto1Dpart();
    ///deactivated since it does not make sense
    Aggreg2Dto1Dpart(const Aggreg2Dto1Dpart<T,Pr>& o);
  public:

    Aggreg2Dto1Dpart(const Pr& p, int nbproc);
    ///it looks like the interval is ]i1:i2] (TODO: Erdeniz can you confirm)
    ///it looks like the interval might be empty and 0 should be returned in that case
    T interval(int i1, int i2) const;

    ~Aggreg2Dto1Dpart ();
  };
}
#include <util/aggreg2dto1dpart_impl.hpp>
#endif
