
// This file is part of SPart, Spatially Located Workload Partitioner.

// Copyright (C) 2011, The Ohio State University
// SPart developed by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek 

// For questions, comments, suggestions, bugs please send e-mail to: 
// Umit V. Catalyurek   catalyurek.1@osu.edu

// SPart is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later
// version.

// SPart is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with SPart; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

// As a special exception, if other files instantiate templates or use
// macros or inline functions from this file, or you compile this file
// and link it with other works to produce a work based on this file,
// this file does not by itself cause the resulting work to be covered by
// the GNU General Public License. However the source code for this file
// must still be made available in accordance with section (3) of the GNU
// General Public License.

// This exception does not invalidate any other reasons why a work based
// on this file might be covered by the GNU General Public License.

// The full text of the GPL license version 2 is available in "gpl.txt".

#include "comm_cost_2d.hpp"

//for debugging purpose only!
template <typename T, typename Pr>
bool util::Comm_cost_2d<T,Pr>::check_neigh(rectangle r1, rectangle r2)
{
  int i;
  //check r1's left
  if(r1.y_top_l!=1)
    if(r1.y_top_l-1 == r2.y_bot_r)
      for(i=r1.x_top_l; i<=r1.x_bot_r;i++)
	if((i<=r2.x_bot_r) && (i>=r2.x_top_l))
	   return true; 
	
  //check r1's right
  if(r1.y_bot_r+1 == r2.y_top_l)
    for(i=r1.x_top_l; i<=r1.x_bot_r;i++)
      if((i<=r2.x_bot_r) && (i>=r2.x_top_l))
	return true;
	   
  //check r1's top
  if(r1.x_top_l!=1)
    if(r1.x_top_l-1 == r2.x_bot_r)
      for(i=r1.y_top_l; i<=r1.y_bot_r;i++)
	if((i<=r2.y_bot_r) && (i>=r2.y_top_l))
	  return true; 
  //check r1's bottom
  if(r1.x_bot_r+1 == r2.x_top_l)
    for(i=r1.y_top_l; i<=r1.y_bot_r;i++)
      if((i<=r2.y_bot_r) && (i>=r2.y_top_l))
	return true;
  return false;
}

template <typename T, typename Pr>
bool util::Comm_cost_2d<T,Pr>::is_neighbor(rectangle r1, rectangle r2)
{
  if(
     (((r1.x_top_l == r2.x_bot_r + 1) ||
       (r1.x_bot_r + 1 == r2.x_top_l)) && y_bounded(r1,r2)) ||
     (((r1.y_top_l == r2.y_bot_r + 1) ||
       (r1.y_bot_r + 1 == r2.y_top_l)) && x_bounded(r1,r2))
     )
    return true;
  else
    return false;
}
template <typename T, typename Pr>
bool util::Comm_cost_2d<T,Pr>::x_bounded(rectangle r1, rectangle r2)
{
  if((r1.x_top_l >= r2.x_top_l) && (r1.x_top_l <= r2.x_bot_r))
    return true;
  else if((r1.x_bot_r >= r2.x_top_l) && (r1.x_bot_r <= r2.x_bot_r))
    return true;
  else if((r2.x_top_l >= r1.x_top_l) && (r2.x_top_l <= r1.x_bot_r))
    return true;
  else if((r2.x_bot_r >= r1.x_top_l) && (r2.x_bot_r <= r1.x_bot_r))
    return true;
  else
    return false;
}
template <typename T, typename Pr>
bool util::Comm_cost_2d<T,Pr>::y_bounded(rectangle r1, rectangle r2)
{
  if((r1.y_top_l >= r2.y_top_l) && (r1.y_top_l <= r2.y_bot_r))
    return true;
  else if((r1.y_bot_r >= r2.y_top_l) && (r1.y_bot_r <= r2.y_bot_r))
    return true;
  else if((r2.y_top_l >= r1.y_top_l) && (r2.y_top_l <= r1.y_bot_r))
    return true;
  else if((r2.y_bot_r >= r1.y_top_l) && (r2.y_bot_r <= r1.y_bot_r))
    return true;
  else
    return false;
}
template <typename T, typename Pr>
double util::Comm_cost_2d<T,Pr>::avg_neighbor(RectList<T, Pr>& rl, int procCount)
{
  
  typename util::RectList<T,Pr>::container::const_iterator list_iter1;
  typename util::RectList<T,Pr>::container::const_iterator list_iter2;
  int total_neighbor=0;
  for (list_iter1 = rl.rectangles.begin(); list_iter1 != rl.rectangles.end(); list_iter1++)
    {
      const rectangle& r1 = *list_iter1;
	  for (list_iter2 = rl.rectangles.begin(); list_iter2 != rl.rectangles.end(); list_iter2++)
	    {
	      const rectangle& r2 = *list_iter2;
	      assert(is_neighbor(r1,r2) == check_neigh(r1,r2));
	      if (list_iter2 == list_iter1)
		continue;
	      if(is_neighbor(r1,r2))
		total_neighbor++;
	    }
    }
  return ((double)total_neighbor)/procCount;
}
template <typename T, typename Pr>
int util::Comm_cost_2d<T,Pr>::max_neighbor(RectList<T, Pr>& rl)
{
  typename util::RectList<T,Pr>::container::const_iterator list_iter1;
  typename util::RectList<T,Pr>::container::const_iterator list_iter2;
  int maximum = 0;
  for (list_iter1 = rl.rectangles.begin(); list_iter1 != rl.rectangles.end(); list_iter1++)
    {
      int neighbor=0;
      const rectangle& r1 = *list_iter1;
	  for (list_iter2 = rl.rectangles.begin(); list_iter2 != rl.rectangles.end(); list_iter2++)
	    {
	      const rectangle& r2 = *list_iter2;
	      assert(is_neighbor(r1,r2) == check_neigh(r1,r2));
	      if (list_iter2 == list_iter1)
		continue;
	      if(is_neighbor(r1,r2))
		neighbor++;
	    }
	  maximum = std::max(maximum,neighbor);
    }
  return maximum;
}




template <typename T, typename Pr>
int util::Comm_cost_2d<T,Pr>::tot_border(RectList<T, Pr>& rl, Pr& prefixSum)
{
  
  typename util::RectList<T,Pr>::container::const_iterator list_iter1;
  typename util::RectList<T,Pr>::container::const_iterator list_iter2;
  int total_length = 0;
  for (list_iter1 = rl.rectangles.begin(); list_iter1 != rl.rectangles.end(); list_iter1++)
    {
      const rectangle& r1 = *list_iter1;
      int lnx=r1.x_bot_r - r1.x_top_l+1;
      if(r1.y_top_l != 1)
	total_length += lnx;
      if(r1.y_bot_r != (prefixSum.prefixsizeY() - 1))
	total_length += lnx; 
      int lny=r1.y_bot_r - r1.y_top_l+1;
      if(r1.x_top_l != 1)
	total_length += lny;
      if(r1.x_bot_r != (prefixSum.prefixsizeX() - 1))
	total_length += lny; 
    }
  return total_length;
}

template <typename T, typename Pr>
int util::Comm_cost_2d<T,Pr>::max_border(RectList<T, Pr>& rl, Pr& prefixSum)
{
  typename util::RectList<T,Pr>::container::const_iterator list_iter1;
  typename util::RectList<T,Pr>::container::const_iterator list_iter2;
  int max_length = 0;
  for (list_iter1 = rl.rectangles.begin(); list_iter1 != rl.rectangles.end(); list_iter1++)
    {
      int total_length = 0;
      const rectangle& r1 = *list_iter1;
      int lnx=r1.x_bot_r - r1.x_top_l+1;
      if(r1.y_top_l != 1)
	total_length += lnx;
      if(r1.y_bot_r != (prefixSum.prefixsizeY() - 1))
	total_length += lnx; 
      int lny=r1.y_bot_r - r1.y_top_l+1;
      if(r1.x_top_l != 1)
	total_length += lny;
      if(r1.x_bot_r != (prefixSum.prefixsizeX() - 1))
	total_length += lny;
      if(total_length > max_length)
	max_length = total_length;
    }
  return max_length;

}
