
// This file is part of SPart, Spatially Located Workload Partitioner.

// Copyright (C) 2011, The Ohio State University
// SPart developed by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek 

// For questions, comments, suggestions, bugs please send e-mail to: 
// Umit V. Catalyurek   catalyurek.1@osu.edu

// SPart is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later
// version.

// SPart is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with SPart; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

// As a special exception, if other files instantiate templates or use
// macros or inline functions from this file, or you compile this file
// and link it with other works to produce a work based on this file,
// this file does not by itself cause the resulting work to be covered by
// the GNU General Public License. However the source code for this file
// must still be made available in accordance with section (3) of the GNU
// General Public License.

// This exception does not invalidate any other reasons why a work based
// on this file might be covered by the GNU General Public License.

// The full text of the GPL license version 2 is available in "gpl.txt".

#ifndef __CONSTRAINT_HPP__
#define __CONSTRAINT_HPP__

namespace util
{
  /**
   * @brief check some properties of a 2d Prefix Sum Array.
   *
   * Namely, it verifies the first row and columns are 0. That every
   * element is positive and that the matrix is non-decreasing
   *
   * @param psa the object to test.
   * @return true if all constraints are valid.
   */
  template < typename Pr>
  bool constraint_psa(const Pr& psa);
  template <typename Pr, typename output>
  void output_psa (const Pr& psa, output& out);

  template < typename Pr>
  bool constraint_psa(const Pr& psa)
  {
    for (int i=0; i < psa.prefixsizeX(); i++)
      if (psa[i][0] != 0) return false;
    
    for (int j=0; j < psa.prefixsizeY(); j++)
      if (psa[0][j] != 0) return false;
    
    for (int i=1; i< psa.prefixsizeX(); i++)
      {
	for (int j=1; j< psa.prefixsizeY(); j++)
	  {
	    if (psa[i][j] < psa[i-1][j]) return false;
	    if (psa[i][j] < psa[i][j-1]) return false;
	    if (psa[i][j] < 0) return false;
	  }
      }
    return true;
  }

  /**
   * @brief outputs the load array represented by a prefix sum array.
   *
   * @param psa a 2d prefix sum array respecting the bracket notation described in util::PrefixSumArray2DOfTBracket
   * @param out a stream the array will be writen to
   */

  template <typename Pr, typename output>
  void output_psa (const Pr& psa, output& out)
  {
    out<<psa.prefixsizeX()-1<<" "<<psa.prefixsizeY()-1<<std::endl;

    for (int i=1; i < psa.prefixsizeX(); i++)
      {
	for (int j=1; j < psa.prefixsizeY(); j++)
	  {
	    out << psa[i][j] + psa[i-1][j-1] - psa[i-1][j] - psa[i][j-1] <<std::endl;
	  }
      }
    
  }
  
}

#endif

