
// This file is part of SPart, Spatially Located Workload Partitioner.

// Copyright (C) 2011, The Ohio State University
// SPart developed by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek 

// For questions, comments, suggestions, bugs please send e-mail to: 
// Umit V. Catalyurek   catalyurek.1@osu.edu

// SPart is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later
// version.

// SPart is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with SPart; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

// As a special exception, if other files instantiate templates or use
// macros or inline functions from this file, or you compile this file
// and link it with other works to produce a work based on this file,
// this file does not by itself cause the resulting work to be covered by
// the GNU General Public License. However the source code for this file
// must still be made available in accordance with section (3) of the GNU
// General Public License.

// This exception does not invalidate any other reasons why a work based
// on this file might be covered by the GNU General Public License.

// The full text of the GPL license version 2 is available in "gpl.txt".

/**
 * @file   multiarrays.hpp
 * @author erdeniz <erdeniz@bmi.osu.edu>
 * @date   Mon Jun 13 12:54:24 2011
 * 
 * @brief A wrapper class to hold multiple arrays
 * 
 * 
 */

#ifndef __MULTIARRAYS_H__
#define __MULTIARRAYS_H__

namespace util
{
  /**
   * @brief A wrapper class to hold multiple arrays
   * 
   */

  template <typename T, typename Pr>
  class Multiarrays
  {
  public:
    /** 
     * @brief Builds a data tructure to hold multiple arrays
     * 
     * @param arrays each element contains a pointer to the head of a 1D array
     * @param lengths each element contains the length of corresponding array
     * @param numArray the number of array in the data structure
     */
    Multiarrays(const Pr ** arrays, const int *lengths, int numArray)
      :arrs(arrays), numArr(numArray), lens(lengths)
    {
      
    }

    /**
     * @return the x-th prefix sum array
     */
    const Pr& operator[](int x) const 
    {
      return *(arrs[x]);
    }
    /**
     * @return the length of the i-th prefix sum array. It is indexed in [0; getLength(i)-1]
     */
    int getLength(int i) const
    {
      return lens[i];
    }
    /**
     * @return the number of prefix sum array
     */
    int arrayCount() const
    {
      return numArr;
    }

  private:
    const Pr** arrs;
    int numArr;
    const int *lens;
  };
}

#endif
