
// This file is part of SPart, Spatially Located Workload Partitioner.

// Copyright (C) 2011, The Ohio State University
// SPart developed by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek 

// For questions, comments, suggestions, bugs please send e-mail to: 
// Umit V. Catalyurek   catalyurek.1@osu.edu

// SPart is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later
// version.

// SPart is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with SPart; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

// As a special exception, if other files instantiate templates or use
// macros or inline functions from this file, or you compile this file
// and link it with other works to produce a work based on this file,
// this file does not by itself cause the resulting work to be covered by
// the GNU General Public License. However the source code for this file
// must still be made available in accordance with section (3) of the GNU
// General Public License.

// This exception does not invalidate any other reasons why a work based
// on this file might be covered by the GNU General Public License.

// The full text of the GPL license version 2 is available in "gpl.txt".

#ifndef __PREFIX_2D__
#define __PREFIX_2D__

namespace util
{

  /**
   * @brief a prefix sum array (2d)
   *
   * Prefix2D respect the 2D prefix sum array skeleton defined by
   * util::PrefixSumArray2DOfTBracket.
   *
   * This class implements 2D prefix sum arrays. From an array A, this
   * class build a prefixsum (let's call it pr) so that $pr[i][j] =
   * \\sum_{i'<i, j'<j} A[i'][j']$
   *
   * @param T data type of Prefix2D
   * 
   */
  template <typename T, bool DEBUG=false>
  class Prefix2D
  {
  private:
    ///this function does not make sense and, thus, is deactivated
    Prefix2D(const Prefix2D& c);
    ///this function does not make sense and, thus, is deactivated
    Prefix2D& operator= (const Prefix2D& c);
    ///this function does not make sense and, thus, is deactivated
    Prefix2D();

    ///size of the actual array, not the prefix sum
    int sizeX;
    ///size of the actual array, not the prefix sum
    int sizeY;
    ///actual memory
    T* data;

  public:
    template<typename ar2d>
    /** 
     * @brief builds the prefix sum of a given array
     * 
     * @param[in] x number of rows in a real array, not prefixSum
     * @param[in] y number of columns in a real array, not prefixSum
     * @param[in] a the 2D array we are building the prefixsum of
     */
    Prefix2D(int x, int y, const ar2d& a);
    /** 
     * 
     * 
     * 
     * @return number of row of the prefixSumArray, i.e., 1 more than
     * the original
     */
    int prefixsizeX() const;
    /** 
     * 
     * 
     * 
     * @return number of column of the prefixSumArray, i.e., 1 more than
     * the original
     */
    int prefixsizeY() const;
    T* operator[] (int x);
    const T* operator[] (int x) const;
    ~Prefix2D();

  };
}

#include "prefix2d_impl.hpp"

#endif
