
// This file is part of SPart, Spatially Located Workload Partitioner.

// Copyright (C) 2011, The Ohio State University
// SPart developed by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek 

// For questions, comments, suggestions, bugs please send e-mail to: 
// Umit V. Catalyurek   catalyurek.1@osu.edu

// SPart is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later
// version.

// SPart is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with SPart; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

// As a special exception, if other files instantiate templates or use
// macros or inline functions from this file, or you compile this file
// and link it with other works to produce a work based on this file,
// this file does not by itself cause the resulting work to be covered by
// the GNU General Public License. However the source code for this file
// must still be made available in accordance with section (3) of the GNU
// General Public License.

// This exception does not invalidate any other reasons why a work based
// on this file might be covered by the GNU General Public License.

// The full text of the GPL license version 2 is available in "gpl.txt".

#include "prefix2d.hpp"
#include "assert.h"

template<typename T, bool DEBUG>
template<typename ar2d>
util::Prefix2D<T,DEBUG>::Prefix2D(int x, int y, const ar2d& a)
    :sizeX(x),sizeY(y)
{
  if (DEBUG)
    assert (! (x<=0 || y<=0));
  
  data = new T[(sizeX+1)*(sizeY+1)];
  for (int i=0; i<=sizeX; i++)
    {
      T sum = 0;
      for (int j=0; j<=sizeY; j++)
	{
	  if (i==0)
	    data[i*(sizeY+1)+j] = 0;
	  else if (j==0)
	    data[i*(sizeY+1)+j] = 0;
	  else
	    {
	      sum += a[i-1][j-1];
	      data[i*(sizeY+1)+j] = data[(i-1)*(sizeY+1)+j] + sum;
	    }
	}
    }
}

template <typename T, bool DEBUG>
int util::Prefix2D<T,DEBUG>::prefixsizeX() const
{
  return sizeX + 1;
}

template <typename T, bool DEBUG>
int util::Prefix2D<T,DEBUG>::prefixsizeY() const
{
  return sizeY + 1;
}

template <typename T, bool DEBUG>
T* util::Prefix2D<T,DEBUG>::operator[] (int x)
{
  if (DEBUG)
    assert(! (x<0 || x > sizeX));
      
  
  return data+(sizeY+1)*x;
}

template <typename T, bool DEBUG>
const T* util::Prefix2D<T,DEBUG>::operator[] (int x) const
{
  if (DEBUG)
    assert(! (x<0 || x > sizeX));
  
  return data+(sizeY+1)*x;
}

template <typename T, bool DEBUG>
util::Prefix2D<T,DEBUG>::~Prefix2D()
{
  delete[] data;
}
