
// This file is part of SPart, Spatially Located Workload Partitioner.

// Copyright (C) 2011, The Ohio State University
// SPart developed by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek 

// For questions, comments, suggestions, bugs please send e-mail to: 
// Umit V. Catalyurek   catalyurek.1@osu.edu

// SPart is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later
// version.

// SPart is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with SPart; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

// As a special exception, if other files instantiate templates or use
// macros or inline functions from this file, or you compile this file
// and link it with other works to produce a work based on this file,
// this file does not by itself cause the resulting work to be covered by
// the GNU General Public License. However the source code for this file
// must still be made available in accordance with section (3) of the GNU
// General Public License.

// This exception does not invalidate any other reasons why a work based
// on this file might be covered by the GNU General Public License.

// The full text of the GPL license version 2 is available in "gpl.txt".

#ifndef __UTIL_TRANSPOSE_PREFIX__
#define __UTIL_TRANSPOSE_PREFIX__

#include "rect_list.hpp"

namespace util{
/**
 * @brief represents the transpose of a prefix sum array.
 *
 * SubPSA respect the 2D prefix sum array skeleton defined by
 * util::PrefixSumArray2DOfTBracket.
 *
 * This class allows to transpose a prefix sum array for
 * free. It assumes that Pr is a prefix sum array of
 * T. It behaves as a regular prefix sum array.
 * @param T type of the element of the matrix
 * @param Pr type of the prefix sum array matrix to transpose 
 */
template <typename T, typename Pr>
class TransposePrefix2D
{
private:
  //those fucntions does not make sense and, thus, are deactivated
  TransposePrefix2D(const TransposePrefix2D& c);
  TransposePrefix2D& operator= (const TransposePrefix2D& c);
  TransposePrefix2D();
  const Pr& originalMatrix;
  

  /**
   * @brief internal data structure of TransposePrefix2D
   */
  class Intermediate
  {
  public:
    const TransposePrefix2D& a;
    int x;
    
    Intermediate(const TransposePrefix2D& ar, int xval)
      :a(ar),x(xval)
    {
    }

    T operator[] (int y) const
    {
      return a.originalMatrix[y][x];
    }
  };

public:
  /** 
   * TransposePrefix2D constructor
   * 
   * @param psa the 2d prefix sum array to transpose
   */
  TransposePrefix2D(const Pr& psa)
    :originalMatrix(psa){}
  /** 
   * @return row number of prefixSumArray, not real array
   */
  int prefixsizeX() const {return originalMatrix.prefixsizeY();}
  /** 
   * 
   * 
   * 
   * @return column number of prefixSumArray, not real array
   */
  int prefixsizeY() const{return originalMatrix.prefixsizeX();}
  //  T* operator[] (int x);
  //const T* operator[] (int x) const;
  Intermediate operator[] (int x) const{return Intermediate(*this,x);}
  ~TransposePrefix2D(){}


  static void convertRectList( const util::RectList<T, util::TransposePrefix2D<T, Pr> > & intermediate, 
			       util::RectList<T,Pr> &parts)
  {
    
    for (typename util::RectList<T,Pr>::container::const_iterator it = intermediate.rectangles.begin();
	 it != intermediate.rectangles.end();
	 it++)
      {
	parts.add_rect(util::rectangle(it->y_top_l,it->y_bot_r,it->x_top_l,it->x_bot_r));
      }

  }
};
}
#endif

