// This file is part of SPart, Spatially Located Workload Partitioner.

// Copyright (C) 2011, The Ohio State University
// SPart developed by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek 

// For questions, comments, suggestions, bugs please send e-mail to: 
// Umit V. Catalyurek   catalyurek.1@osu.edu

// SPart is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 3 or (at your option) any later
// version.

// SPart is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with SPart; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

// The full text of the GPL license version 3 is available in "gpl.txt".


#include <stdio.h>
#include <stdlib.h>
#include <assert.h>


//
//let c_0 = 0 and c_{p+1} = n
//returns max_j \sum_{i \in ]c_{j-1}:c_j]} array[j]

/** 
 * Synopsis: check_1dpart <instance_file> <proc_num> < <cut_point_file>
 * cut_point_file: read on the standard input cut points c_1, c_2, c_3, ...
 * instance_file: Begins with a line with number of tasks and the rest contains 1 task load per line. 
 * Calculates the bottleneck of a given instance an cut points
 * @param argc 
 * @param argv 
 * 
 * @return bottleneck
 */
int main (int, char* argv[])
{
  int numProc = atoi(argv[2]), arraySize;
  double *array;
  int *cutIndexes;

  double max;
  int index;
  int p;

  //reading instance file
  {
    FILE* f;
    int i;
    f = fopen (argv[1], "r");
    fscanf(f, "%d", &arraySize);
    array = (double*)malloc(sizeof(double)*(arraySize));
    cutIndexes = (int*)malloc(sizeof(int)*(numProc-1));
    for(i = 0; i < arraySize; i++)
      {
	fscanf(f, "%lf", &array[i]);
      }           
    fclose(f);
  }
  
  index = 0;
  max = 0;
  //read cut points from std input
  for (p=1;p<=numProc; p++)
    {
      double sum=0;
      int cut;
      scanf("%d\n", &cut);

      assert (cut<=arraySize-1);

      for (; index<=cut; index++)
	{
	  assert (index<arraySize);
	  sum += array[index];
	}
      
      max = (sum>max?sum:max);
    }

  {
    double sum=0;
    for (; index<arraySize; index++)
      {
	sum += array[index];
      }
    max = (sum>max?sum:max);
  }
  

  printf ("%lf\n", max);


  return 0;


}
