
// This file is part of SPart, Spatially Located Workload Partitioner.

// Copyright (C) 2011, The Ohio State University
// SPart developed by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek 

// For questions, comments, suggestions, bugs please send e-mail to: 
// Umit V. Catalyurek   catalyurek.1@osu.edu

// SPart is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later
// version.

// SPart is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with SPart; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

// The full text of the GPL license version 2 is available in "gpl.txt".

#include <stdio.h>
#include <string.h>
#include "oned/nicol_minus.hpp"
#include "oned/nicol.hpp"
#include "oned/nicol_plus.hpp"


int main(int, char* argv[])
{
  using namespace oned;
  //double a1[] = {0,1,2,3,4, 5, 6, 7, 8, 9, 10};
  double w1[] = {0,1,3,6,10,15,21,28,36,45,55};
  int n1 = 11;
  //int p1 = 4;
  //int cut[10];
  double bottleneck;


  //double a2[] = {0,1,2,3, 4, 5};
  double w2[] = {0,1,3,6,10,15};
  int n2 = 6;

  double (*tested_fnct)(int,  double* const& , int , int* , double);

  if (strcmp(argv[1],"nicol_minus") == 0)
    {
      tested_fnct = NicolMinus<double, double* >::nicol_minus;
    }
  else
    if (strcmp(argv[1],"nicol") == 0)
    {
  tested_fnct = Nicol<double, double* >::nicol;
    }
  else if (strcmp(argv[1],"nicol_plus") == 0)
    {
      tested_fnct = NicolPlus<double, double* >::nicol_plus;
    }

  else
    {fprintf (stderr,"must select an algorithm\n");return -1;}


  bottleneck = tested_fnct(1, w2,n2, NULL, -1);

  if(bottleneck != 15)
    {
      fprintf(stderr, "Error in a2/w2 partitioning in 1 processors. returns %lf instead of 15\n", bottleneck);
      return -1;
    }

  bottleneck = tested_fnct(2, w2,n2,NULL, -1);

  if(bottleneck != 9)
    {
      fprintf(stderr, "Error in a2/w2 partitioning in 2 processors. returns %lf instead of 9\n", bottleneck);
      return -1;
    }

  bottleneck = tested_fnct(3, w2,n2, NULL, -1);

  if(bottleneck != 6)
    {
      fprintf(stderr, "Error in a2/w2 partitioning in 3 processors. returns %lf instead of 6\n", bottleneck);
      return -1;
    }

  bottleneck = tested_fnct(1, w1,n1, NULL, -1);

  if(bottleneck != 55)
    {
      fprintf(stderr, "Error in a1/w1 partitioning in 1 processors. returns %lf instead of 55\n", bottleneck);
      return -1;
    }

  bottleneck = tested_fnct(2, w1,n1, NULL, -1);

  if(bottleneck != 28)
    {
      fprintf(stderr, "Error in a1/w1 partitioning in 2 processors. returns %lf instead of 28\n", bottleneck);
      return -1;
    }

  bottleneck = tested_fnct(3, w1,n1, NULL, -1);

  if(bottleneck != 21)
    {
      fprintf(stderr, "Error in a1/w1 partitioning in 3 processors. returns %lf instead of 21\n", bottleneck);
      return -1;
    }

  bottleneck = tested_fnct(4, w1,n1, NULL, -1);

  if(bottleneck != 17)
    {
      fprintf(stderr, "Error in a1/w1 partitioning in 4 processors. returns %lf instead of 17\n", bottleneck);
      return -1;
    }
  return 0;
}
