
// This file is part of SPart, Spatially Located Workload Partitioner.

// Copyright (C) 2011, The Ohio State University
// SPart developed by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek 

// For questions, comments, suggestions, bugs please send e-mail to: 
// Umit V. Catalyurek   catalyurek.1@osu.edu

// SPart is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later
// version.

// SPart is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with SPart; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

// The full text of the GPL license version 2 is available in "gpl.txt".

#include "util/prefix_sum_tools.hpp"
#include "util/transpose2d.hpp"
#include <stdlib.h>
#include <stdio.h>
#include <iostream>
void btest()
{
  using namespace util;
  double arr[4000];
  double pr[4001];
  int i;
  int c;
  for (i=0; i< 2000; i++)
    {
      arr[i] = 0;
    }
  for (;i<4000;i++)
    {
      arr[i] = 4;
    }
  prefixsum(arr, pr, 4000);

  if ((c = binarySearch (pr, 0, 4000, 8.)) != 2002)
    {
      fprintf(stderr, "error btest() binarysearch return %d instead of 2002\n",c);
    }

}

int main ()
{
  using namespace util;
  int a1Length = 10; //,a2Length = 15;
  double a1[] = {1,2,3,4,5,6,7,8,9,10};
  double pa[] = {0,1,3,6,10,15,21,28,36,45,55};
  //  double a2[] = {3,4,5,3,6,1,6,5,3,8,7,3,7,3,9};
  //int *cutIndexes;
  int i, j;
  //  int cuts1[3] = {4,6,8};
  //int cuts2[2] = {5,7};
  double *a1prefixSum = new double[a1Length+1];
  prefixsum(a1,a1prefixSum, a1Length);

  for (i=0; i< a1Length+1; i++)
    {
      if (pa[i] != a1prefixSum[i])
	{
	  fprintf(stderr, "error in prefix sum array computation\n");
	  return -1;
	}      
    }

  btest();

  if (binarySearch (a1prefixSum, 0, a1Length, 2.) != 1)
    {
      fprintf(stderr, "binary search fail on 2\n");
      return -1;
    }
  if (binarySearch (a1prefixSum, 0, a1Length, 16.) != 5)
    {
      fprintf(stderr, "binary search fail on 16\n");
      return -1;
    }

  if (binarySearch (a1prefixSum, 0, a1Length, 15.) != 5)
    {
      fprintf(stderr, "binary search fail on 15\n");
      return -1;
    }


  if (binarySearch (a1prefixSum, 0, a1Length, 1.) != 1)
    {
      fprintf(stderr, "binary search fail on 1\n");
      return -1;
    }


  if (binarySearch (a1prefixSum, 0, a1Length, 55.) != 10)
    {
      fprintf(stderr, "binary search fail on 55\n");
      return -1;
    }


  if (binarySearch (a1prefixSum, 0, a1Length, 56.) != 10)
    {
      fprintf(stderr, "binary search fail on 56\n");
      return -1;
    }


  if (binarySearch (a1prefixSum, 0, a1Length, 0.) != 0)
    {
      fprintf(stderr, "binary search fail on 0\n");
      return -1;
    }
  delete[] a1prefixSum;
  
  double data[4][4] = {{1,2,3,4},
		       {5,6,7,8},
		       {9,10,11,12},
		       {13,14,15,16}};
  double expected_pre[5][5] = {{0,0,0,0,0},
			       {0,1,3,6,10},
			       {0,6,14,24,36},
			       {0,15,33,54,78},
			       {0,28,60,96,136}};
  Prefix2D<double> pr(4, 4, data);
  for(i = 0; i < 5; i++)
      for(j=0; j < 5; j++)
	if(pr[i][j] != expected_pre[i][j])
	  {
	    fprintf(stderr, "2d prefix sum builder fail\n");
	    return -1;
	  }

  TransposePrefix2D<double, Prefix2D<double> > tpr (pr);
  for(i = 0; i < 5; i++)
      for(j=0; j < 5; j++)
	if(pr[i][j] != tpr[j][i])
	  {
	    fprintf(stderr, "transpose fail\n");
	  }

  {  

    double data2[4][3] = {{1,2,3},
			  {5,6,7},
			  {9,10,11},
			  {13,14,15}};
    Prefix2D<double> pr(4, 3, data2);
    TransposePrefix2D<double, Prefix2D<double> > tpr (pr);

    assert (pr.prefixsizeX() == tpr.prefixsizeY());
    assert (pr.prefixsizeY() == tpr.prefixsizeX());

    for(i = 0; i < tpr.prefixsizeX(); i++)
      for(j = 0; j < tpr.prefixsizeY(); j++)
	if(tpr[i][j] != pr[j][i])
	  {
	    fprintf(stderr, "transpose fail\n");
	  }
  }
  


  return 0;
}
