
// This file is part of SPart, Spatially Located Workload Partitioner.

// Copyright (C) 2011, The Ohio State University
// SPart developed by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek 

// For questions, comments, suggestions, bugs please send e-mail to: 
// Umit V. Catalyurek   catalyurek.1@osu.edu

// SPart is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later
// version.

// SPart is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with SPart; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

// The full text of the GPL license version 2 is available in "gpl.txt".

#include "twod/her_jag_2d.hpp"
#include <iostream>
#include "util/rect_list.hpp"
#include "oned/nicol_plus.hpp"

int main()
{
  using namespace oned;
  using namespace twod;
  using namespace util;
  rectangle r;
  int result;
  int array1[4][4],array2[10][10], i, j;
  for(i=0;i<4;i++)
    for(j=0;j<4;j++)
      array1[i][j]=4*i+j;
 
  Prefix2D<int> prefixSum(4,4,array1);
  {
    RectList<int,Prefix2D<int> > rl1(prefixSum);
    JagPQHeurHor<int,Prefix2D<int>,      NicolPlus<int, Aggreg2Dto1D<int, Prefix2D<int>, false> >::nicol_plus,
      NicolPlus<int, Aggreg2Dto1D<int, Prefix2D<int>,  true> >::nicol_plus > foo;
    result = foo.part(4, prefixSum, rl1);
    if((result != 39) || !rl1.valid_part())
      {
	std::cerr<<"error in her jag 2d rl1 part 1"<<std::endl;
	return -1;
      }
  }
  {
    RectList<int,Prefix2D<int> > rl2(prefixSum);
    JagPQHeurHor<int,Prefix2D<int>,       NicolPlus<int, Aggreg2Dto1D<int, Prefix2D<int>, false> >::nicol_plus,
      NicolPlus<int, Aggreg2Dto1D<int, Prefix2D<int>,  true> >::nicol_plus> foo;
    result = foo.part(4, prefixSum, rl2);
    if((result != 39) || !rl2.valid_part())
      {
	std::cerr<<"error in her jag 2d rl1 part 2 " <<std::endl;
	return -1;
      }
  }
  for(i=0;i<10;i++)
    for(j=0;j<10;j++)
      array2[i][j]=10*i+j;

  Prefix2D<int> prefixSum2(10,10,array2);
  {
    RectList<int,Prefix2D<int> > rl3(prefixSum2);
    JagPQHeurHor<int,Prefix2D<int>,   NicolPlus<int, Aggreg2Dto1D<int, Prefix2D<int>, false> >::nicol_plus,
      NicolPlus<int, Aggreg2Dto1D<int, Prefix2D<int>,  true> >::nicol_plus> foo;
    result = foo.part(9, prefixSum2, rl3);
    if((result != 692) || !rl3.valid_part())
      {
	std::cerr<<"error in her jag 2d rl3 part"<<std::endl;
	return -1;
      }
  }
  
  return 0;
}
