
// This file is part of SPart, Spatially Located Workload Partitioner.

// Copyright (C) 2011, The Ohio State University
// SPart developed by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek 

// For questions, comments, suggestions, bugs please send e-mail to: 
// Umit V. Catalyurek   catalyurek.1@osu.edu

// SPart is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later
// version.

// SPart is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with SPart; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

// The full text of the GPL license version 2 is available in "gpl.txt".

#include "twod/nicol_2d.hpp"
#include <iostream>
#include "util/rect_list.hpp"
#include "oned/nicol_plus.hpp"

int main()
{
  using namespace twod;
  using namespace util;
    int result;
    int array1[4][4],
    array2[7][7] = {{0,0,0,1,0,0,0},
                    {0,0,0,0,0,0,1},
                    {1,1,1,1,1,1,0},
                    {0,0,1,0,0,0,0},
                    {0,1,1,0,1,1,1},
                    {0,1,0,1,1,1,1},
                    {0,0,1,0,1,1,1}
    };
    int i, j;
    for (i=0;i<4;i++)
        for (j=0;j<4;j++)
            array1[i][j]=4*i+j;

    Prefix2D<int> prefixSum(4,4,array1);
    int ycuts[] = {2};
    AggregMax2Dto1D<int,Prefix2D<int>,true> max2d_1(prefixSum, 2, ycuts);
    if(max2d_1.interval(0,2) != 18)
      {
	std::cerr<<"error in nicols2d max2d_1 is " << max2d_1.interval(0,2)<<std::endl;
	return -1;
      }
    if(max2d_1.interval(1,2) != 13)
      {
	std::cerr<<"error in nicols2d max2d_1 is " << max2d_1.interval(1,2)<<std::endl;
	return -1;
      }
    int *xcuts=ycuts;
    AggregMax2Dto1D<int,Prefix2D<int>,false> max2d_2(prefixSum, 2, xcuts);
    if(max2d_2.interval(0,2) != 42)
      {
	std::cerr<<"error in nicols2d max2d_2 is " << max2d_2.interval(0,2)<<std::endl;
	return -1;
      }
    if(max2d_2.interval(1,2) != 22)
      {
	std::cerr<<"error in nicols2d max2d_2 is " << max2d_2.interval(1,2)<<std::endl;
	return -1;
      }
    {
      RectList<int,Prefix2D<int> > rl1(prefixSum);
      RectNicol<int,Prefix2D<int> > foo;
      result = foo.part(4, prefixSum, rl1);
      if ((result != 39) || !rl1.valid_part())
	{
	  std::cerr<<"error in nicol 2d rl1 part. Should be 39 but is "<<result<<std::endl;
	  std::cerr<<"valid partition?  "<<rl1.valid_part()<<std::endl;
	  return -1;
	}
    }

    {
      Prefix2D<int> prefixSum2(7,7,array2);
      RectList<int,Prefix2D<int> > rl3(prefixSum2);
      RectNicol<int,Prefix2D<int> > foo;
      result = foo.part(9, prefixSum2, rl3);
      if ((result != 4)  ||  !rl3.valid_part() )
	{
	  std::cerr<<"error in nicol 2d rl3 part. Should be 4 but "<< result<<std::endl;
	  std::cerr<<"valid partition?  "<<rl3.valid_part()<<std::endl;
	  return -1;
	}
    }

    return 0;
}
