
// This file is part of SPart, Spatially Located Workload Partitioner.

// Copyright (C) 2011, The Ohio State University
// SPart developed by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek 

// For questions, comments, suggestions, bugs please send e-mail to: 
// Umit V. Catalyurek   catalyurek.1@osu.edu

// SPart is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later
// version.

// SPart is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with SPart; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

// The full text of the GPL license version 2 is available in "gpl.txt".

#include <stdio.h>
#include <stdlib.h>
#include "oned/rec_bisect.hpp"

int main()
{
  using namespace oned;
  double wpre1[] = {0,1,3,6,10,15,21,28,36,45,55};
  double wpre2[] = {0,5,12,15,28,55,83,94,106,460,540,600,740,801,906,1024,1029,1036,3800,4000,4050};
  double wpre3[] = {0,3,456,750,900,1000,1400,1500,1550,2000,2100,2142,2321,2400,2410,2412};
  double wpre4[] = {0,1,2,3,1000,1001,1002,1003};
  int numProc1 = 2, numProc2 = 3, numProc3 = 5, numProc4 = 4;
  int cuts1[2], cuts2[6], cuts3[numProc3-1],cuts4[numProc4-1];
  int length1 = 11, length2 = 21, length3 = 16, length4 = 8;
  double val;

  val = RecursiveBisection<double, double* >::rec_bisection(numProc1, wpre1, length1,  cuts1, -1.);
  if (val != 28 || cuts1[0] != 7)
    {
      fprintf(stderr,"cut1 values: %lf cut: %d\n", val, cuts1[0]);
      fprintf(stderr,"Error in cut1 partition\n");
      return -1;
    }
  
  numProc1=3;
  val = RecursiveBisection<double, double* >::rec_bisection(numProc1, wpre1, length1,  cuts1, -1.);
  if(val != 21 || (cuts1[0] != 6) || (cuts1[1] != 8))
    {
      fprintf(stderr,"cut5 val = %lf cuts: %d and %d\n", val, cuts1[0], cuts1[1]);
      fprintf(stderr,"Error in cut5 partition\n");
      return 1;
    }
  RecursiveBisection<double, double* >::rec_bisection(numProc2, wpre2, length2, cuts2, -1.);
  if((cuts2[0] != 17) || (cuts2[1] != 18))
    {
      fprintf(stderr,"Error in cut2 partition cuts: %d %d\n", cuts2[0], cuts2[1]);
      return 1;
    }
  RecursiveBisection<double, double* >::rec_bisection(numProc3, wpre3, length3,  cuts3, -1.);
  if((cuts3[0] != 2) || (cuts3[1] != 5) || (cuts3[2] != 7) || (cuts3[3] != 9))
    {
      fprintf(stderr,"Error in cut3 partition cuts: %d %d %d %d\n", cuts3[0], cuts3[1],cuts3[2],cuts3[3]);
      return 1;
    }
  RecursiveBisection<double, double* >::rec_bisection(numProc4, wpre4, length4,  cuts4, -1.);
  if((cuts4[0] != 3) || (cuts4[1] != 4) || (cuts4[2] != 6))
    {
      fprintf(stderr,"Error in cut4 partition cuts: %d %d %d\n", cuts4[0], cuts4[1],cuts4[2]);
      return 1;
    }
  return 0;
}
