
// This file is part of SPart, Spatially Located Workload Partitioner.

// Copyright (C) 2011, The Ohio State University
// SPart developed by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek 

// For questions, comments, suggestions, bugs please send e-mail to: 
// Umit V. Catalyurek   catalyurek.1@osu.edu

// SPart is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later
// version.

// SPart is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with SPart; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

// The full text of the GPL license version 2 is available in "gpl.txt".

#include "twod/rec_bisect_2d.hpp"
#include "util/rect_list.hpp"
#include <iostream>

int test_rec_bisect()
{
  using namespace twod;
  using namespace util;
  rectangle r;
  double result;
  int array1[4][4], array2[10][10], i, j;
  for(i=0;i<4;i++)
    for(j=0;j<4;j++)
      array1[i][j]=4*i+j;
 
  Prefix2D<double> prefixSum(4,4,array1);
  {
    RectList<double,Prefix2D<double> > rl1(prefixSum);
    RecBisect2D<double,Prefix2D<double>,2> foo;
    result = foo.part(4, prefixSum, rl1);
    if(!rl1.valid_part() || result != 39)
      {
	std::cerr << "2d rect bisect error 1 should be 39 found: " << result << std::endl;
	return -1;
      }
  }

  for(i=0;i<10;i++)
    for(j=0;j<10;j++)
      array2[i][j]=10*i+j;

  Prefix2D<double> prefixSum2(10,10,array2);
  {
    RectList<double,Prefix2D<double> > rl2(prefixSum2);
    RecBisect2D<double,Prefix2D<double>, RecBisect2D<double,Prefix2D<double> >::VERT_ALT> foo;
    result = foo.part(8, prefixSum2, rl2);
    if(!rl2.valid_part() || result != 774)
      {
	std::cerr << "2d rect bisect error 2 " << result <<std::endl;
	return -1;
      }
  }

  {
    RectList<double,Prefix2D<double> > rl3(prefixSum2);
    RecBisect2D<double,Prefix2D<double>,RecBisect2D<double,Prefix2D<double> >::VERT_ALT> foo;
    result = foo.part(3, prefixSum2, rl3);
    if(!rl3.valid_part() || result != 1743)
      {
	std::cerr << "2d rect bisect error 3 " << result <<std::endl;
	return -1;
      }
  }
  return 0;
}
int test_load_rec()
{
  using namespace twod;
  using namespace util;
  rectangle r;
  double result;
  int array1[4][4], array2[10][10], i, j;
  for(i=0;i<4;i++)
    for(j=0;j<4;j++)
      array1[i][j]=4*i+j;

  Prefix2D<double> prefixSum(4,4,array1);

  {
    RectList<double,Prefix2D<double> > rl1(prefixSum);
    RecBisect2D<double,Prefix2D<double>,RecBisect2D<double,Prefix2D<double> >::LOAD> foo;
    result = foo.part(4, prefixSum, rl1);
    if(!rl1.valid_part() || result != 38)
      {
	std::cerr << "2d load rect bisect error 1" << std::endl;
	return -1;
      }
  }

  for(i=0;i<10;i++)
    for(j=0;j<10;j++)
      array2[i][j]=10*i+j;

  Prefix2D<double> prefixSum2(10,10,array2);
  {
    RectList<double,Prefix2D<double> > rl2(prefixSum2);
    RecBisect2D<double,Prefix2D<double>,RecBisect2D<double,Prefix2D<double> >::LOAD> foo;
    result = foo.part(8, prefixSum2, rl2);
    if(!rl2.valid_part() || result != 774)
      {
	std::cerr << "2d load rect bisect error 2 " << result <<std::endl;
	return -1;
      }
  }
  {
    RectList<double,Prefix2D<double> > rl3(prefixSum2);
    RecBisect2D<double,Prefix2D<double>, RecBisect2D<double,Prefix2D<double> >::LOAD> foo;
    result = foo.part(3, prefixSum2, rl3);
    if(!rl3.valid_part() || result != 1743)
      {
	std::cerr << "2d load rect bisect error 3 " << result <<std::endl;
	return -1;
      }
  }
  return 0;
}

int test_dist_rec()
{
  using namespace twod;
  using namespace util;
  rectangle r;
  double result;
  int array1[4][4], array2[10][10], array3[4][16], i, j;
  for(i=0;i<4;i++)
    for(j=0;j<4;j++)
      array1[i][j]=4*i+j;

  Prefix2D<double> prefixSum(4,4,array1);
  {
    RectList<double,Prefix2D<double> > rl1(prefixSum);
    RecBisect2D<double,Prefix2D<double>,RecBisect2D<double,Prefix2D<double> >::DIST> foo;
    result = foo.part(4, prefixSum, rl1);
    if(!rl1.valid_part() || result != 39)
      {
	std::cerr << "2d dist rect bisect error 1" << std::endl;
	return -1;
      }
  }

  for(i=0;i<10;i++)
    for(j=0;j<10;j++)
      array2[i][j]=10*i+j;

  Prefix2D<double> prefixSum2(10,10,array2);
  {
    RectList<double,Prefix2D<double> > rl2(prefixSum2);
    RecBisect2D<double,Prefix2D<double>,RecBisect2D<double,Prefix2D<double> >::DIST> foo;
    result = foo.part(8, prefixSum2, rl2);
    if(!rl2.valid_part() || result != 774)
      {
	std::cerr << "2d dist rect bisect error 2 " << result <<std::endl;
	return -1;
      }
  }
  {
    RectList<double,Prefix2D<double> > rl3(prefixSum2);
    RecBisect2D<double,Prefix2D<double>,RecBisect2D<double,Prefix2D<double> >::DIST> foo;
    result = foo.part(3, prefixSum2, rl3);
    if(!rl3.valid_part() || result != 1743)
      {
	std::cerr << "2d dist rect bisect error 3 " << result <<std::endl;
	return -1;
      }
  }
  for(i=0;i<4;i++)
    for(j=0;j<16;j++)
      array3[i][j]=16*i+j;

  Prefix2D<double> prefixSum3(4,16,array3);
  {
    RectList<double,Prefix2D<double> > rl4(prefixSum3);
    RecBisect2D<double,Prefix2D<double>,RecBisect2D<double,Prefix2D<double> >::DIST> foo;
    result = foo.part(4, prefixSum3, rl4);
    if(!rl4.valid_part() || result != 552)
      {
	std::cerr << "2d dist rect bisect error 4" << std::endl;
	return -1;
      }
  }
  return 0;
}

int main()
{
  bool orth_based = test_rec_bisect();
  bool load_based = test_load_rec();
  bool dist_based = test_dist_rec();

  if( orth_based || load_based || dist_based)
    return -1;
  
  return 0;
}
