
// This file is part of SPart, Spatially Located Workload Partitioner.

// Copyright (C) 2011, The Ohio State University
// SPart developed by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek 

// For questions, comments, suggestions, bugs please send e-mail to: 
// Umit V. Catalyurek   catalyurek.1@osu.edu

// SPart is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later
// version.

// SPart is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with SPart; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

// The full text of the GPL license version 2 is available in "gpl.txt".

#include "prefix_sum_tools.hpp"
#include "rec_bisect_minmax.hpp"
#include <iostream>

double pointDiff(int *prefixSum, int low, int high, int leftProc, int rightProc, int point)
{
  double y1 = RecursiveBisectionMinMax<int,int*>::calculateDiff (prefixSum, low, high, leftProc, rightProc, point - 1);
  double y2 = RecursiveBisectionMinMax<int,int*>::calculateDiff (prefixSum, low, high, leftProc, rightProc, point);
  return y2-y1;
}

//-1 means we are in decreasing side
//+1 means we are in increasing side
int slopeAt(int *prefixSum, int low, int high, int leftProc, int rightProc, int point)
{

  double diff = pointDiff(prefixSum, low, high, leftProc, rightProc, point);
  if(diff <0)
    return -1;
  else if(diff>0)
    return 1;
  else
    {
      //if slope is zero at that point
      while((low<point) && (diff == 0) )
	{
	  point--;
	  diff = pointDiff(prefixSum, low, high, leftProc, rightProc, point);
	}
      if(diff <= 0)
	return -1;
      else
	return 1;
      
    }
	  
  
}
int bimonsearch(int *prefixSum, int low, int high, int leftProc, int rightProc)
{
  assert(low<high);
  int lowlimit=low;
  int highlimit=high;
  int mid;
  while(lowlimit<highlimit)
    {
      mid = (lowlimit+highlimit)/2;
      //std::cout<<"mid value:"<<mid<<std::endl;
      //std::cout<<"slope value:"<<slopeAt(prefixSum, low, high,  leftProc, rightProc,  mid)<<std::endl;
      if(slopeAt(prefixSum, low, high,  leftProc, rightProc,  mid) == -1)
	{
	  lowlimit = mid+1;
	}
      else
	highlimit = mid;
      
    }
  return lowlimit;
}
int main()
{
  int array[] = {4,8,17,7,6,7,19,5,14,16};
  int array2[] = {1,1,1,1,1,1,1,1,1,1};
  int *prefixSum = new int[11];
  int *prefixSum2 = new int[11];
  int i;
  prefixsum(array, prefixSum, 10);
  prefixsum(array2, prefixSum2, 10);
  for(i=2;i<11;i++)
    std::cout<<"slope " << i << "point is: "<<slopeAt(prefixSum, 1, 10, 1, 1, i)<<std::endl;
  for(i=2;i<11;i++)
    std::cout<<"slope " << i << "point is: "<<slopeAt(prefixSum2, 1, 10, 1, 1, i)<<std::endl;
  std::cout << "mid point of prefixsum is: "<<bimonsearch(prefixSum, 1, 10, 1, 1)<<std::endl;
  std::cout << "mid point of prefixsum2 is: "<<bimonsearch(prefixSum2, 1, 10, 1, 1)<<std::endl;
  std::cout<< prefixSum[bimonsearch(prefixSum, 1, 10, 1, 1) - 1] <<std::endl;
  std::cout<< prefixSum2[bimonsearch(prefixSum2, 1, 10, 1, 1) - 1] <<std::endl;
}
