
// This file is part of SPart, Spatially Located Workload Partitioner.

// Copyright (C) 2011, The Ohio State University
// SPart developed by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek 

// For questions, comments, suggestions, bugs please send e-mail to: 
// Umit V. Catalyurek   catalyurek.1@osu.edu

// SPart is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later
// version.

// SPart is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with SPart; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

// The full text of the GPL license version 2 is available in "gpl.txt".

#include "util/prefix_sum_tools.hpp"
#include "util/subpsa.hpp"
#include <iostream>
#include <fstream>

int main ()
{
  using namespace util;
  std::istream* input;

  int sizeX, sizeY;

  char* filename = "regression_test/data/2d/repartition_5000";
  if (true)
    {
      //read from file
      input = new std::ifstream(filename);
    }

  (*input)>>sizeX>>sizeY;

  //reading the input
  Compact2D<long int> data(sizeX, sizeY);
  for (int i=0; i<sizeX; i++)
    for (int j=0; j<sizeY; j++)
      (*input)>>data[i][j];


  if (true)
    {
      delete input;
    }
  
  Prefix2D<long int, true> pr(sizeX, sizeY, data);

  {
    SubPSA<long int, Prefix2D<long int, true> > spsa (pr, rectangle(1,514,1,514));
  
    assert (pr.prefixsizeX() == spsa.prefixsizeX());
    assert (pr.prefixsizeY() == spsa.prefixsizeY());
    
    for (int i=0; i<= 514; i++)
      for (int j=0; j<514; j++)
	assert (pr[i][j] == spsa[i][j]);
  }

  {
    SubPSA<long int, Prefix2D<long int, true> > spsa (pr, rectangle(1,514,1,514));
  
    assert (pr.prefixsizeX() == spsa.prefixsizeX());
    assert (pr.prefixsizeY() == spsa.prefixsizeY());
    
    for (int i=0; i<= 514; i++)
      for (int j=0; j < 514; j++)
	assert (pr[i][j] == spsa[i][j]);
  }


  {
    rectangle r(1, 257, 130, 161);
    SubPSA<long int, Prefix2D<long int, true> > spsa (pr, r);
  
    for (int i=0; i<spsa.prefixsizeX(); i++)
      assert (spsa[i][0] == 0);
  
    for (int j=0; j<spsa.prefixsizeY(); j++)
      assert (spsa[0][j] == 0);
    
    rectangle r2 (1, spsa.prefixsizeX()-1,
		  1, spsa.prefixsizeY()-1);

    long int rvalue = r.get_load<long int, Prefix2D<long int, true> >(pr);
    long int r2value = r2.get_load<long int, SubPSA<long int, Prefix2D<long int, true> > >(spsa);
    assert ( rvalue == r2value );
  }



  return 0;
}
