
// This file is part of SPart, Spatially Located Workload Partitioner.

// Copyright (C) 2011, The Ohio State University
// SPart developed by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek 

// For questions, comments, suggestions, bugs please send e-mail to: 
// Umit V. Catalyurek   catalyurek.1@osu.edu

// SPart is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later
// version.

// SPart is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with SPart; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

// The full text of the GPL license version 2 is available in "gpl.txt".

#include "util/compact.hpp"
#include <string.h>
#include <iostream>
#include <fstream>
#include <string>
#include <assert.h>
#include <util/timestamp.hpp>
#include <twod/parse.hpp>
#include "util/rect_list.hpp"
#include "util/comm_cost_2d.hpp"
#include "twod/hybrid.hpp"
#include "twod/part_base.hpp"
#include <vector>

#ifdef HAVE_BOOST
#include <boost/iostreams/filtering_stream.hpp>
#include <boost/iostreams/filter/gzip.hpp>
#include <boost/iostreams/filter/bzip2.hpp>
#endif

int main(int argc, char* argv[])
{
  using namespace oned;
  using namespace twod;
  using namespace util;
  



  //HEADER START
  std::cerr<<"=================================================================="<<std::endl;
  std::cerr<<"SPart, Spatially Located Workload Partitioner. Version "<<SPART_VERSION<<std::endl;
  std::cerr<<"=================================================================="<<std::endl;
  std::cerr<<"Copyright (C) 2011, The Ohio State University\ndeveloped by Erdeniz O. Bas, Erik Saule and Umit V. Catalyurek\nThis is free software with ABSOLUTELY NO WARRANTY.\nRun with no parameter for usage and licensing information."<<std::endl;
  //HEADER END

  if (argc != 7)
    {
      std::cerr<<std::endl;

      std::cerr<<"usage : "<<argv[0]<<" <algoLevel1> <algoLevel2> <algoLevel2Slow> <sizeLevel1> <inst> <nbproc>"<<std::endl;

      std::cerr<<std::endl;

      std::cerr<<"LICENSING AND WARRANTY"<<std::endl<<std::endl;
      std::cerr<<"SPart is free software; you can redistribute it and/or modify it under"<<std::endl;
      std::cerr<<"the terms of the GNU General Public License as published by the Free"<<std::endl;
      std::cerr<<"Software Foundation; either version 2 or (at your option) any later"<<std::endl;
      std::cerr<<"version."<<std::endl;
      std::cerr<<""<<std::endl;
      std::cerr<<"SPart is distributed in the hope that it will be useful, but WITHOUT"<<std::endl;
      std::cerr<<"ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or"<<std::endl;
      std::cerr<<"FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License"<<std::endl;
      std::cerr<<"for more details."<<std::endl;
      std::cerr<<""<<std::endl;
      std::cerr<<"You should have received a copy of the GNU General Public License"<<std::endl;
      std::cerr<<"along with SPart; if not, write to the Free Software Foundation, Inc.,"<<std::endl;
      std::cerr<<"51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA."<<std::endl;

      return -1;
    }

  int sizeX, sizeY;


  typedef double T;
  typedef Prefix2D<T> Pr;

  twod::PartBase<T, Pr >* pb = NULL;

  std::string algo1 = argv[1];
  std::string algo2 = argv[2];
  std::string algo2slow = argv[3];

  int s1 = atoi(argv[4]);

  int procCount = atoi(argv[6]);

  Hybrid<T,Pr>* h = new Hybrid<T,Pr>();

  h->setLevel1(Parser<T,Pr>::parseName(algo1));
  h->setLevel2(Parser<T, util::SubPSA<T,Pr> > ::parseName(algo2) );
  h->setLevel2Slow(Parser<T, util::SubPSA<T,Pr> > ::parseName(algo2slow) );
  h->setProcLevel1(s1);
		
  pb = h;

#ifdef HAVE_BOOST
  boost::iostreams::filtering_stream<boost::iostreams::input> input;
  
  std::ifstream foo ;
  
  if(strcmp(argv[5], "-") == 0)
    {
      //read from standard input
      //      input = &std::cin;
      input.push (std::cin);
    }
  else
    {
      //read from file
      
      foo.open (argv[5]);
      if (!foo.is_open())
	{
	  std::cerr<< "can not open the instance file: "<<argv[5]<<std::endl;
	  exit(-1);
	}
      {
	int len = strlen(argv[5]);
	if (len > 3 && strcmp (argv[5]+len-3, ".gz") == 0)
	  input.push(boost::iostreams::gzip_decompressor());
	if (len > 4 && strcmp (argv[5]+len-4, ".bz2") == 0)
	  input.push(boost::iostreams::bzip2_decompressor());
	
      }
      
      input.push (foo);
      //input = foo;
    }
  
  input>>sizeX>>sizeY;
  
  //reading the input
  Compact2D<T> data(sizeX, sizeY);
  for (int i=0; i<sizeX; i++)
    for (int j=0; j<sizeY; j++)
      input>>data[i][j];
  
  if (input.bad() || input.fail())
    {
      std::cerr<<"improper load matrix !"<<std::endl;
      return -1;
    }
  
  
  if(strcmp(argv[5], "-") == 0)
    {
      //      input = &std::cin;
    }
  else
    {
      //delete input;
      foo.close();
    }
  
#else
  std::istream* input;
  
  std::ifstream foo ;
  
  if(strcmp(argv[5], "-") == 0)
    {
      //read from standard input
      input = & std::cin;
    }
  else
    {
      //read from file
      
      foo.open (argv[5]);
      if (!foo.is_open())
	{
	  std::cerr<< "can not open the instance file: "<<argv[5]<<std::endl;
	  exit(-1);
	}
      
      //      input.push (foo);
      input = &foo;
    }
  
  (*input)>>sizeX>>sizeY;
  
  //reading the input
  Compact2D<T> data(sizeX, sizeY);
  for (int i=0; i<sizeX; i++)
    for (int j=0; j<sizeY; j++)
      (*input)>>data[i][j];
  
  if (input->bad() || input->fail())
    {
      std::cerr<<"improper load matrix !"<<std::endl;
      return -1;
    }

  if(strcmp(argv[5], "-") == 0)
    {
      //      input = &std::cin;
    }
  else
    {
      //delete input;
      foo.close();
    }
#endif
  

  Pr pr(sizeX, sizeY, data);



  RectList<T, Pr > rl(pr);

  timestamp t1;

  T d = pb->part(procCount, pr, rl);

  timestamp t2;

  delete pb;

  assert (rl.valid_part());

  util::RectList<T,Pr>::container::const_iterator list_iter1 = rl.rectangles.begin();
  for(;list_iter1 != rl.rectangles.end(); list_iter1++)
    {
      const rectangle &r =  *list_iter1;
      std::cout << r.x_top_l  <<  " " << r.y_top_l << " ";
      std::cout << r.x_bot_r  <<  " " << r.y_bot_r << std::endl;
    }

  std::cerr<<"time             (in seconds)        : "<<t2-t1<<std::endl;
  std::cerr<<"most loaded part                     : "<<d<<std::endl;
  std::cerr<<"load imbalance   ((max-ideal)/ideal) : "<<((double)d-(double)(pr[sizeX][sizeY])/procCount)/(pr[sizeX][sizeY]/procCount)<<std::endl;

  // std::cerr<<"average neighbors : "<<Comm_cost_2d<T, Pr >::avg_neighbor(rl,procCount)<<std::endl;
  // std::cerr<<"maximum neighbors : "<<Comm_cost_2d<T, Pr >::max_neighbor(rl)<<std::endl;
  // std::cerr<<"total border length : "<<Comm_cost_2d<T, Pr >::tot_border(rl,pr)<<std::endl;
  // std::cerr<<"max border length : "<<Comm_cost_2d<T, Pr >::max_border(rl,pr)<<std::endl;
  return 0;
}
